package uk.ac.ic.doc.y2002.tool.application;

import java.net.URL;
import java.io.IOException;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;
import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;
import java.lang.reflect.*;
import uk.ac.ic.doc.y2002.tool.editor.*;

/** GUI for Petri-net Tool
 */
public class Application extends JFrame {
    public JDesktopPane desktop; // desktop of application.
    public String subnetRef = null;
    public Element subnetRoot = null;
    private MyButton newButton, openButton, saveButton, printButton, cutButton, copyButton, pasteButton;
    private MySelectButton arcButton, pointerButton, placeButton, refPlaceButton, immediateButton, timedButton, addtokButton,
	removetokButton, subnetButton, animateButton, pauseButton, revertButton, refButton;
    private JSlider slider;
    private MyMenuItem pauseItem, revertItem; // here because need to control enabled/disabled state
    private JToolBar toolBar;
    JMenuItem[] runModule = new JMenuItem[6];
    JMenuItem[] removeModule = new JMenuItem[6];
    Object[] module = new Object[6];
    public static Object currentModule;
    MenuLoadModuleListener loadModuleListener = new MenuLoadModuleListener(this);

    public static String setTool = "pointer";              // selected tool.
    public static final String IMAGES = "./icons/";       // the image resource path
    public static Clipboard myClipboard = new Clipboard(); // selectionbox for cut and paste etc.
    boolean animateOn=false;

    // Constructor for class GUI
    public Application() {

	super("EXPT (EXtensible Petri-net Tool)");

	// create components for application
	Container contentPane = getContentPane();
	contentPane.setLayout( new BorderLayout() );
	contentPane.add(createDesktop(), BorderLayout.CENTER);
	createMenuToolBar();

	// set default window size.
	int inset = 100;
	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	setBounds(inset, inset, screenSize.width - inset*2, screenSize.height - inset*2);

	// set default tool icon
	pointerButton.select();

	// add window listener
	ApplicationFrameListener windowListener = new ApplicationFrameListener();
	addWindowListener(windowListener);
	setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    }

    // Function to create DeskTop
    private JDesktopPane createDesktop() {

	desktop = new JDesktopPane();
	desktop.setBackground(Color.white);

	return desktop;
    }

    // Function to create menuBar and toolBar
    private void createMenuToolBar() {

	// Listeners for file menu
	MenuOpenListener openListener =  new MenuOpenListener(this);
	MenuNewListener newListener = new MenuNewListener(this, desktop);
	//MenuNewSubnetListener newSubnetListener = new MenuNewSubnetListener(this,desktop);
	MenuSaveListener saveListener = new MenuSaveListener(desktop);
	MenuSaveAsListener saveAsListener = new MenuSaveAsListener(desktop);
	MenuExitListener exitListener = new MenuExitListener(desktop);

	// Edit menus
	MenuEditCutListener cutListener = new MenuEditCutListener(desktop);
	MenuEditPasteListener pasteListener = new MenuEditPasteListener(desktop);
	MenuEditCopyListener copyListener = new MenuEditCopyListener(desktop);

	// listeners for pointer, place tools etc.
	ToolBarListener toolListener = new ToolBarListener(this);
	MenuInsertListener insertListener = new MenuInsertListener(this);
	AnimateListener animateListener = new AnimateListener(this, desktop);
	SliderListener sliderListener = new SliderListener(this, desktop);

	// Start of MenuBar construction
	JMenuBar menuBar = new JMenuBar();

	JMenu fileMenu = new JMenu("File");
	JMenu editMenu = new JMenu("Edit");
	JMenu insertMenu = new JMenu("Insert");
	JMenu moduleMenu = new JMenu("Modules");
	JMenu animationMenu = new JMenu("Animation");
	JMenu helpMenu = new JMenu("Help");

	fileMenu.setMnemonic('F');
	editMenu.setMnemonic('E');
	insertMenu.setMnemonic('I');
	moduleMenu.setMnemonic('M');
	animationMenu.setMnemonic('A');
	helpMenu.setMnemonic('H');

	// file menu
	MyMenuItem my_new = new MyMenuItem("New", 'N', 'N',"newnet");
	MyMenuItem new_subnet = new MyMenuItem("New Subnet", 'w', 'M',"newsubnet");
	MyMenuItem open = new MyMenuItem("Open", 'O', 'O');
	MyMenuItem save = new MyMenuItem("Save", 'S', 'S');
	MyMenuItem saveAs = new MyMenuItem("Save as..", 0, 0);
	MyMenuItem exit = new MyMenuItem("Exit", 'E', 0);

	open.addActionListener(openListener);
	my_new.addActionListener(newListener);
	new_subnet.addActionListener(newListener);
	save.addActionListener(saveListener);
	saveAs.addActionListener(saveAsListener);
	exit.addActionListener(exitListener);

	fileMenu.add(my_new);
	fileMenu.add(new_subnet);
	fileMenu.addSeparator();
	fileMenu.add(open);
	fileMenu.add(save);
	fileMenu.add(saveAs);
	fileMenu.addSeparator();
	fileMenu.add(exit);

	// edit menu
	MyMenuItem cut = new MyMenuItem("Cut", 'u', 'X', "cut");
	MyMenuItem copy = new MyMenuItem("Copy", 'C', 'C', "copy");
	MyMenuItem paste = new MyMenuItem("Paste", 'P', 'V', "paste");
	MyMenuItem delete = new MyMenuItem("Delete", 'D', 0, "delete");
	delete.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_DELETE, 0));

	cut.addActionListener(cutListener);
	copy.addActionListener(copyListener);
	paste.addActionListener(pasteListener);
	delete.addActionListener(cutListener);

	editMenu.add(cut);
	editMenu.add(copy);
	editMenu.add(paste);
	editMenu.addSeparator();
	editMenu.add(delete);

	// insert menu
	MyMenuItem pointerMenuItem = new MyMenuItem("Pointer", IMAGES + "pointer.gif", "pointer", 'n');
	MyMenuItem placeMenuItem = new MyMenuItem("Place", IMAGES + "place1.gif", "place", 'P');
	MyMenuItem refPlaceMenuItem = new MyMenuItem("Reference Place", IMAGES + "refPlace1.gif", "refPlace", 'f');
	MyMenuItem immediateMenuItem  = new MyMenuItem("Immediate Transition", IMAGES + "immediate1.gif",
						       "immediate", 'I');
	MyMenuItem timedMenuItem = new MyMenuItem("Timed Transition", IMAGES + "timed1.gif", "timed", 'T');
	MyMenuItem arcMenuItem = new MyMenuItem("Arc", IMAGES + "arc1.gif", "arc", 'A');
	MyMenuItem refArcMenuItem = new MyMenuItem("Reference Arc", IMAGES + "ref1.gif", "ref", 'e');
	MyMenuItem addtokMenuItem = new MyMenuItem("Add Token", IMAGES + "addtok1.gif", "addtok", 'O');
	MyMenuItem removetokMenuItem = new MyMenuItem("Remove Token", IMAGES + "remtok1.gif", "remtok", 'R');
	MyMenuItem subnetMenuItem = new MyMenuItem("Subnet", IMAGES + "subnet1.gif", "subnet", 'S');

	pointerMenuItem.addActionListener(insertListener);
	placeMenuItem.addActionListener(insertListener);
	refPlaceMenuItem.addActionListener(insertListener);
	immediateMenuItem.addActionListener(insertListener);
	timedMenuItem.addActionListener(insertListener);
	arcMenuItem.addActionListener(insertListener);
	refArcMenuItem.addActionListener(insertListener);
	addtokMenuItem.addActionListener(insertListener);
	removetokMenuItem.addActionListener(insertListener);
	subnetMenuItem.addActionListener(insertListener);

	insertMenu.add(placeMenuItem);
	insertMenu.add(refPlaceMenuItem);
	insertMenu.add(immediateMenuItem);
	insertMenu.add(timedMenuItem);
	insertMenu.add(arcMenuItem);
	insertMenu.add(refArcMenuItem);
	insertMenu.add(addtokMenuItem);
	insertMenu.add(removetokMenuItem);
	insertMenu.add(subnetMenuItem);

    MyMenuItem loadModule = new MyMenuItem("Load a Module", 'L', 'L');
	JMenu runModuleSubMenu = new JMenu("Run a Module");
	JMenu removeModuleSubMenu = new JMenu("Remove a Module");
	runModuleSubMenu.setMnemonic('R');
	removeModuleSubMenu.setMnemonic('e');
        loadModule.addActionListener(loadModuleListener);
        for(int i=0; i < runModule.length; i++) {
	    runModule[i] = new JMenuItem();
	    runModuleSubMenu.add(runModule[i]);
	    runModule[i].setVisible(false);

	    removeModule[i] = new JMenuItem();
	    removeModuleSubMenu.add(removeModule[i]);
	    removeModule[i].setVisible(false);

	    module[i] = null;
        }
        removeModule[0].addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    module[0] = null;
		    runModule[0].setVisible(false);
		    removeModule[0].setVisible(false);
		}
	    });
        removeModule[1].addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    module[1] = null;
		    runModule[1].setVisible(false);
		    removeModule[1].setVisible(false);
		}
	    });
        removeModule[2].addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    module[2] = null;
		    runModule[2].setVisible(false);
		    removeModule[2].setVisible(false);
		}
	    });
        removeModule[3].addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    module[3] = null;
		    runModule[3].setVisible(false);
		    removeModule[3].setVisible(false);
		}
	    });
        removeModule[4].addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    module[4] = null;
		    runModule[4].setVisible(false);
		    removeModule[4].setVisible(false);
		}
	    });
        removeModule[5].addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    module[5] = null;
		    runModule[5].setVisible(false);
		    removeModule[5].setVisible(false);
		}
	    });
        // runModuleMenuitems
        runModule[0].addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    currentModule = module[0];
		    loadModuleListener.runAModule();

		}
	    });
        runModule[1].addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    currentModule = module[1];
		    loadModuleListener.runAModule();
		}
	    });
        runModule[2].addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    currentModule = module[2];
		    loadModuleListener.runAModule();
		}
	    });
        runModule[3].addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    currentModule = module[3];
		    loadModuleListener.runAModule();
		}
	    });
        runModule[4].addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    currentModule = module[4];
		    loadModuleListener.runAModule();
		}
	    });
        runModule[5].addActionListener(new java.awt.event.ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    currentModule = module[5];
		    loadModuleListener.runAModule();
		}
	    });

        moduleMenu.add(loadModule);
        moduleMenu.addSeparator();
        moduleMenu.add(runModuleSubMenu);
        moduleMenu.add(removeModuleSubMenu);


	// animation menu
	MyMenuItem animateItem = new MyMenuItem("Animate", 'A', 'A', "animate");
	pauseItem = new MyMenuItem("Pause", 'P', 0, "pause"); //declared at top
	revertItem = new MyMenuItem("Stop", 'S', 0, "revert"); // declared at top

	animateItem.addActionListener(animateListener);
	pauseItem.addActionListener(animateListener);
	revertItem.addActionListener(animateListener);
	pauseItem.setEnabled(false);
	revertItem.setEnabled(false);

	animationMenu.addSeparator();
	animationMenu.add(animateItem);
	animationMenu.add(pauseItem);
	animationMenu.add(revertItem);


	// help menu.
	MyMenuItem about = new MyMenuItem("About us..", 'A', 0);
	about.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    JFrame aboutFrame = new JFrame("EXPT");
		    JPanel panel = new JPanel();
		    panel.setBorder(BorderFactory.createLoweredBevelBorder());
		                    String text = "<html>\n" +
					        "We are :" +
					        "<ul>\n" +
					        "<li><font color=blue>Raphael Goldberg</font>\n" +
					        "<li><font color=blue>Peter Howarth</font>\n" +
					        "<li><font color=blue>Daniel Justice</font>\n" +
					        "<li><font color=blue>Michelle Osmond</font>\n" +
					        "<li><font color=blue>Zhu Tan</font>\n" +
					        "<li><font color=blue>Alexander Trifunovic</font></ul>\n" +
					        "And our tool is: <p>"+
					    "<font color=green size=+2><i>EXtensible Petri net Tool</i></font>\n<p>" +
					"(Written at Imperial college 2001-2002)";
				    JLabel content = new JLabel(text);
				    panel.add(content);
				    aboutFrame.getContentPane().add(panel, "Center");
				    aboutFrame.setSize(400,300);
				    aboutFrame.setVisible(true);
		}
	    });

	// help menu user guide.
	MyMenuItem guide = new MyMenuItem("User guide", 'U', 0);
        guide.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent event) {
		    final JEditorPane editor = new JEditorPane();
		    try {
			String s = null;
			s = "file:"
			    + System.getProperty("user.dir")
			    + System.getProperty("file.separator")
			    + "UserGuide.htm";
			URL helpURL = new URL(s);
			/* ...  use the URL to initialize the editor pane  ... */
			editor.setEditable(false);
			editor.setContentType("text/html");
			editor.setPage(helpURL);
		    } catch (Exception e) { System.out.println("Application::UserGuide.htm not found!"); }

		    JFrame guideFrame = new JFrame("User guide");
		    guideFrame.getContentPane().add(new JScrollPane(editor));
		    guideFrame.setSize(700,700);
		    guideFrame.setVisible(true);
		}
	    });

	helpMenu.add(about);
	helpMenu.add(guide);

	// add menus to menuBar
	menuBar.add(fileMenu);
	menuBar.add(editMenu);
	menuBar.add(insertMenu);
	menuBar.add(moduleMenu);
	menuBar.add(animationMenu);
	menuBar.add(helpMenu);

	// Start of JToolBar
	toolBar = new JToolBar();

	newButton = new MyButton(IMAGES + "filenew.gif", "New", "newnet" );
	openButton = new MyButton(IMAGES + "fopen.gif", "Open", "open" );
	saveButton = new MyButton(IMAGES + "fsave.gif", "Save", "save" );
	cutButton = new MyButton(IMAGES + "cut.gif", "Cut", "cut");
	copyButton = new MyButton(IMAGES + "copy.gif", "Copy", "copy");
	pasteButton = new MyButton(IMAGES + "paste.gif", "Paste", "paste" );
	pointerButton = new MySelectButton(IMAGES + "pointer1.gif", IMAGES + "pointer2.gif", "Pointer", "pointer" );
	placeButton = new MySelectButton(IMAGES + "place1.gif", IMAGES + "place2.gif", "Place", "place" );
	refPlaceButton = new MySelectButton(IMAGES + "refPlace1.gif", IMAGES + "refPlace2.gif", "Reference place", "refPlace" );
	immediateButton = new MySelectButton(IMAGES + "immediate1.gif", IMAGES + "immediate2.gif", "Immediate transition", "immediate" );
	timedButton = new MySelectButton(IMAGES + "timed1.gif", IMAGES + "timed2.gif", "Timed transition", "timed" );
	arcButton = new MySelectButton(IMAGES + "arc1.gif", IMAGES + "arc2.gif", "Arc", "arc");
	refButton = new MySelectButton(IMAGES + "ref1.gif", IMAGES + "ref2.gif", "Reference Link", "ref");
	addtokButton = new MySelectButton(IMAGES + "addtok1.gif", IMAGES + "addtok2.gif", "Add token", "addtok");
	removetokButton = new MySelectButton(IMAGES + "remtok1.gif", IMAGES + "remtok2.gif",  "Remove token", "remtok");
	subnetButton = new MySelectButton(IMAGES + "subnet1.gif", IMAGES + "subnet2.gif", "Subnet", "subnet");
	animateButton = new MySelectButton(IMAGES + "animate1.gif", IMAGES + "animate2.gif", "Animate", "animate");
	pauseButton = new MySelectButton(IMAGES + "pause1.gif", IMAGES + "pause2.gif", "Pause", "pause");
	pauseButton.setEnabled(false);
	revertButton = new MySelectButton(IMAGES + "stop1.gif", IMAGES + "stop2.gif", "Revert", "revert");
	revertButton.setEnabled(false);
	slider = new JSlider(SwingConstants.HORIZONTAL, 100,2000,1000);
	slider.setInverted(true);
	slider.setToolTipText("Adjust animation speed");
	slider.setEnabled(false);

	newButton.addActionListener(newListener);
	openButton.addActionListener(openListener);
	saveButton.addActionListener(saveListener);
	cutButton.addActionListener(cutListener);
	copyButton.addActionListener(copyListener);
	pasteButton.addActionListener(pasteListener);
	pointerButton.addActionListener(toolListener);
	placeButton.addActionListener(toolListener);
	refPlaceButton.addActionListener(toolListener);
	immediateButton.addActionListener(toolListener);
	timedButton.addActionListener(toolListener);
	arcButton.addActionListener(toolListener);
	refButton.addActionListener(toolListener);
	addtokButton.addActionListener(toolListener);
	removetokButton.addActionListener(toolListener);
	subnetButton.addActionListener(toolListener);
	animateButton.addActionListener(animateListener);
	pauseButton.addActionListener(animateListener);
	revertButton.addActionListener(animateListener);
	slider.addChangeListener(sliderListener);

	toolBar.add(newButton);
	toolBar.add(openButton);
	toolBar.add(saveButton);
	toolBar.addSeparator();
	toolBar.add(cutButton);
	toolBar.add(copyButton);
	toolBar.add(pasteButton);
	toolBar.addSeparator();
	toolBar.add(pointerButton);
	toolBar.add(placeButton);
	toolBar.add(refPlaceButton);
	toolBar.add(immediateButton);
	toolBar.add(timedButton);
	toolBar.add(arcButton);
	toolBar.add(refButton);
	toolBar.add(addtokButton);
	toolBar.add(removetokButton);
	toolBar.add(subnetButton);
	toolBar.addSeparator();
	toolBar.add(animateButton);
	toolBar.add(pauseButton);
	toolBar.add(revertButton);
	toolBar.add(slider);
	toolBar.setFloatable(false);

	// Add MenuBar and ToolBar to Application.
	setJMenuBar(menuBar);
	Container contentPane = getContentPane();
	contentPane.add(toolBar, BorderLayout.NORTH);
    }

    // Sets the currently selected tool and update button image.
    public void chooseTool(String button_cmd) {

	// un-select all buttons.
	pointerButton.unSelect();
	placeButton.unSelect();
	refPlaceButton.unSelect();
	immediateButton.unSelect();
	timedButton.unSelect();
	arcButton.unSelect();
	refButton.unSelect();
	addtokButton.unSelect();
	removetokButton.unSelect();
	subnetButton.unSelect();
	animateButton.unSelect();
	pauseButton.unSelect();
	revertButton.unSelect();

	// set command for selected button.
	setTool = button_cmd;

	// set selected button item.
	MySelectButton button = chooseSelectedButton(button_cmd);
	button.select();
    }

    /** This function enables/disables buttons appropriately,
     * given an animation state.
     */
    public void setAnimate(boolean inSubnet, boolean animating){
	boolean notanimating = !animating;

	toolBar.repaint();
	// disable/enable buttons.
	cutButton.setEnabled(notanimating);
	copyButton.setEnabled(notanimating);
	pasteButton.setEnabled(notanimating);
	pointerButton.setEnabled(notanimating);
	placeButton.setEnabled(notanimating);
	refPlaceButton.setEnabled(notanimating);
	immediateButton.setEnabled(notanimating);
	timedButton.setEnabled(notanimating);
	arcButton.setEnabled(notanimating);
	refButton.setEnabled(notanimating);
	addtokButton.setEnabled(notanimating);
	removetokButton.setEnabled(notanimating);
	subnetButton.setEnabled(notanimating);
	// animateButton: always enabled unless frame is a subnet instance
	animateButton.setEnabled(!inSubnet);
	pauseButton.setEnabled(animating);
	revertButton.setEnabled(animating);
	slider.setEnabled(animating);

	pauseItem.setEnabled(animating);
	revertItem.setEnabled(animating);

    }

    // internal classes
    /** A window listener */

    protected class ApplicationFrameListener extends WindowAdapter{

	public ApplicationFrameListener(){
	    super();
	}

	public void windowClosing(WindowEvent e) {

	    int response = JOptionPane.showConfirmDialog(desktop, new String[] {
		"Do you really want to quit?"}, "Close application",
		JOptionPane.YES_NO_OPTION);

	    switch(response) {
	    case JOptionPane.YES_OPTION: System.exit(0);
	    case JOptionPane.NO_OPTION: // don't quit
	    }
	}
    }

    // mimics the action of a user pressing save
    public void simulateSave() {
	saveButton.doClick();
    }


    /** This function retrieves the current animation state,
     * and enables/disables buttons appropriately.
     */
    public void setAnimate(boolean inSubnet){
	setAnimate(inSubnet, isAnimating());
    }

    public boolean isAnimating(){
	return getSelectedDisplayPane().animator.isAnimating();
    }
    public int getTimestep(){
	return slider.getValue();
    }

    public EditorFrame getSelectedFrame(){
	EditorFrame my_frame = (EditorFrame) desktop.getSelectedFrame();
	return my_frame;
    }

    public DisplayPane getSelectedDisplayPane(){
	EditorFrame eframe = getSelectedFrame();
	DisplayPane dpane;
	if (eframe!=null) {
	    dpane = eframe.display;
	    return dpane;
	} else return null;
    }

    public MySelectButton chooseSelectedButton(String button_cmd) {

	if (button_cmd.equals("place"))
	    return placeButton;
	else if (button_cmd.equals("refPlace"))
	    return refPlaceButton;
	else if (button_cmd.equals("immediate"))
	    return immediateButton;
	else if (button_cmd.equals("timed"))
	    return timedButton;
	else if (button_cmd.equals("arc"))
	    return arcButton;
	else if (button_cmd.equals("ref"))
	    return refButton;
	else if (button_cmd.equals("addtok"))
	    return addtokButton;
	else if (button_cmd.equals("remtok"))
	    return removetokButton;
	else if (button_cmd.equals("subnet")){
	    //choose the subnet to insert
	    if(chooseSubnet());{
		return subnetButton;
	    }
	}
	else if (button_cmd.equals("animate"))
	    return animateButton;
	else if (button_cmd.equals("pause"))
	    return pauseButton;
	else if (button_cmd.equals("revert"))
	    return revertButton;
	// else
	return pointerButton;
    }

    private boolean chooseSubnet() {
	java.util.List editorFrames;
	editorFrames = java.util.Arrays.asList((JInternalFrame[]) desktop.getAllFrames());
	ArrayList subnets = new ArrayList();
	ArrayList roots = new ArrayList();
	ArrayList paths = new ArrayList();
	Iterator ei = editorFrames.iterator();
	while(ei.hasNext()){
	    System.out.println("checking a frame");
	    EditorFrame ef = (EditorFrame)ei.next();
	    if (ef.root.getName().equals("module")){
		String name = ef.file.getName() + "#" + ef.root.getAttributeValue("name");
		subnets.add(name);
		paths.add(ef.file.getPath() + "#" + ef.root.getAttributeValue("name"));
		roots.add(ef.root);
	    }
	}
	if(subnets.isEmpty()) {
	    System.out.println("No subnets found");
	    JOptionPane.showMessageDialog(desktop, "No subnets found.\nPlease open a subnet template.",
					  "Subnet opener",JOptionPane.ERROR_MESSAGE, null);
	    return false;
	}
	Object input;
	input = JOptionPane.showInternalInputDialog(desktop,"Choose a subnet template to add",
						    "New subnet - choose template",
						    JOptionPane.QUESTION_MESSAGE,null,subnets.toArray(),subnets.get(0));

	if (input==null) {
	    // user selected Cancel
	    return false;
	}

	// find the selected option and set the subnet reference
	int index = subnets.indexOf(input);
	SubnetLabel.chosenRef = paths.get(index).toString();
	SubnetLabel.chosenRoot = (Element)roots.get(index);
	System.out.println(SubnetLabel.chosenRoot.getName());
	return true;
    }

    public static void main(String[] args) {
	try {
            UIManager.setLookAndFeel(
				     UIManager.getSystemLookAndFeelClassName() );
	    System.out.println("Application:: set look and feel");
	} catch (Exception e) {
	    System.out.println("Application:: can't set look and feel");
	}

	Application app = new Application();
	app.setVisible(true);
    }

}




