
package uk.ac.ic.doc.y2002.tool.application;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.awt.geom.*;
import java.io.*;
import java.lang.reflect.*;
import org.jdom.*;
import org.jdom.output.*;
import uk.ac.ic.doc.y2002.tool.editor.*;

public class DisplayPane extends JPanel {
    public EditorFrame eframe;
    public Element root;
    public Namespace ns;
    public ClassManager classman;
    Dimension preferredSize = new Dimension(500,500);
    PNLabel source = null, target = null;   // used to identify source and target in arc creation.
    public PNAnimator animator;
    HashMap idTable = new HashMap(); // Hash table to store ids in use.
    HashMap placeTable = new HashMap(); // Hash table for places.
    HashMap referencePlaceTable = new HashMap(); // Hash table for referencePlaces.
    HashMap transitionTable = new HashMap(); // transitions.
    HashMap subnetTable = new HashMap(); // subnets.
    HashMap arcTable = new HashMap(); // arcs.
    public HashMap refLinkTable = new HashMap(); // used to map sourceID -> refLinkLabel
    private Clipboard myClipboard = Application.myClipboard; // link to static selectionbox in Application.
    public SelectionBox mySelection = null;  
    HashMap idmapping = new HashMap(); // used for paste and edit functions.
    private String moduleRef = null;
    public java.util.List refPairs;   // variable to store reference link source and targets 
    Class propertiesbox, contextmenu;
    boolean updated = true;        // boolean used to determine whether a petri-net has been udpated.

    /** On creation, the DisplayPane, should be passed
     * the <net> or <module> element which it is to display.
     */
    public DisplayPane(EditorFrame eframe, Element root){
	//System.out.println("DisplayPane::initialising...");
	setLayout(null);
	this.root = root;
	this.eframe = eframe;
	ns = root.getNamespace();
	loadClassManager(root);
	animator = classman.newAnimator(this);
	//---create and add Labels---//
	makeLabels();
	calcPreferredSize();
	//System.out.println("DisplayPane::initialised.");
	
	// Listeners for panel
	DisplayPaneListener paneListener = new DisplayPaneListener();
	//KeyEventListener keyListener = new KeyEventListener();
	//      addKeyListener(keyListener);
	
	addMouseListener(paneListener);
	addMouseMotionListener(paneListener);
	
	propertiesbox = DisplayPropertiesBox.class;
	contextmenu = PetriContextMenu.class;
    }

    public void loadClassManager(Element root){
	//System.out.println("DisplayPane::loadClassManager()");
	String type = root.getAttributeValue("type");
	classman = new ClassManager(type);
    }

    public void makeLabels() {
	//System.out.println("DisplayPane::makeLabels()");
	refPairs = new ArrayList();
	
	idTable.clear();
	placeTable.clear();
	referencePlaceTable.clear();
	transitionTable.clear();
	subnetTable.clear();
	arcTable.clear();
	refLinkTable.clear();

	// update this later to deal with changeable Classes
	java.util.List places = root.getChildren("place",ns);
	Iterator ps= places.iterator();
	String id;
	
	boolean loadedPropertiesBox=false;

	while (ps.hasNext()) {    
	    //    System.out.println("place loop");
	    Element p = (Element) ps.next();
	    PlaceLabel label = classman.newPlaceLabel(p);
	    //    tgt = label;
	    id = p.getAttributeValue("id");
	    // add unique id to hashSet and update placeTable
	    idTable.put(id, label);
	    placeTable.put(id, label);
	    // namespace check?
	    add(label);
    
	    if (!loadedPropertiesBox)
		new PropertiesFrame(label.returnElement());
	}
	
	java.util.List transitions = root.getChildren("transition",ns);
	Iterator ts= transitions.iterator();
	while (ts.hasNext()) {
	    //    System.out.println("transition loop");
	    Element t = (Element) ts.next();
	    TransitionLabel label = classman.newTransitionLabel(t);
	    id = t.getAttributeValue("id");
	    // add unique id to hashSet and update Transitiontable
	    idTable.put(id, label);
	    transitionTable.put(id, label);
	    // namespace check?
	    add(label);
	}
	
	java.util.List subnets = root.getChildren("instance",ns);
	Iterator is = subnets.iterator();
	while (is.hasNext()) {
	    Element t = (Element) is.next();
	    SubnetLabel label = classman.newSubnetLabel(t);
	    id = t.getAttributeValue("id");
	    idTable.put(id, label);
	    subnetTable.put(id, label);
	    add(label); 
	    // check to see if import place has a reference
	    Element ip = t.getChild("importPlace", ns);
	    String src = t.getAttributeValue("id") + "." + ip.getAttributeValue("parameter");
	    String tgt = null;
	    if (ip.getAttributeValue("instance") == null){
		tgt = ip.getAttributeValue("ref");
	    }
	    else {
		tgt = ip.getAttributeValue("instance") + "." + ip.getAttributeValue("ref");
	    }
	    //System.out.println(src + tgt);
	    if (!(tgt==null)) {
		refPairs.add(src);
		refPairs.add(tgt);
	    }
	     
	}

	
	java.util.List referencePlaces = root.getChildren("referencePlace",ns);
	Iterator rs= referencePlaces.iterator();
	while (rs.hasNext()) {
	    Element r = (Element) rs.next();
	    ReferencePlaceLabel label = classman.newReferencePlaceLabel(r);
	    id = r.getAttributeValue("id");
	    // add unique id to hashSet and update placeTable
	    idTable.put(id, label);
	    referencePlaceTable.put(id, label);
	    add(label);
	    // Create reference link pairs
	    String src = r.getAttributeValue("id");
	    String tgt = null;
	    if (r.getAttributeValue("instance") == null){
		tgt = r.getAttributeValue("ref");
	    }
	    else {
		tgt = r.getAttributeValue("instance") + "." + r.getAttributeValue("ref");
	    }
	    if (!(tgt==null)) {
		refPairs.add(src);
		refPairs.add(tgt);
	    }
	}
	
	java.util.List interfaces = root.getChildren("interface",ns);
	Iterator ints= interfaces.iterator();
	while (ints.hasNext()) {
	    //System.out.println("Entering the interface loop");
	    Element i = (Element) ints.next();
	                        
	    //Create the importplaces
	    java.util.List importPlaces = i.getChildren("importPlace",ns);
	    Iterator ips= importPlaces.iterator();
	    while (ips.hasNext()) {
		Element j = (Element) ips.next();
		ImportPlaceLabel label = classman.newImportPlaceLabel(j);
		id = j.getAttributeValue("id");
		// add unique id to hashSet and update placeTable
		idTable.put(id, label);
		referencePlaceTable.put(id, label);
		// namespace check?
		add(label);
		// Create reference link pairs
		String src = j.getAttributeValue("id");
		String tgt = j.getAttributeValue("ref");
		if (!(tgt==null)) {
		    refPairs.add(src);
		    refPairs.add(tgt);
		}

	    }
	    //Create the exportplaces
	    java.util.List exportPlaces = i.getChildren("exportPlace",ns);
	    Iterator eps= exportPlaces.iterator();
	    while (eps.hasNext()) {
		//System.out.println("Entering the exportPlace loop");
		Element k = (Element) eps.next();
		ExportPlaceLabel label = classman.newExportPlaceLabel(k);
		id = k.getAttributeValue("id");
		// add unique id to hashSet and update placeTable
		idTable.put(id, label);
		referencePlaceTable.put(id, label);
		// namespace check?
		add(label);
		// Create reference link pairs
		String src = k.getAttributeValue("id");
		String tgt = k.getAttributeValue("ref");
		if (!(tgt==null)) {
		    refPairs.add(src);
		    refPairs.add(tgt);
		}
	    }
	                            
	}
	// create the RefLinkLabels, valid pairs should be in the refPairs list 
	Iterator rl = refPairs.iterator(); 
	while (rl.hasNext()) {
	    try{
		source = getLabel((String)rl.next());
		target = getLabel((String)rl.next());
		RefLinkLabel label = new RefLinkLabel(source, target);
		// 0 means add label to front of container
		add(label, 0);
		// add refNode target and sources.
		NodeLabel my_source = (NodeLabel) source;
		NodeLabel my_target = (NodeLabel) target;
		my_source.addRefNodeTarget(my_target);
		my_target.addRefNodeSource(my_source);
		// add source ID refLinkLabel to hashmap
		refLinkTable.put(my_source, label);

	    }catch(Exception e) {System.out.println("Exception: " + e); }
	}
	
	// should be done last, so that other elements can 
	// be connected together
	java.util.List arcs = root.getChildren("arc",ns);
	Iterator as= arcs.iterator();
	while (as.hasNext()) {
	    //System.out.println("arc loop");
	    Element a = (Element) as.next();
	    ArcLabel label = classman.newArcLabel(a);
	    id = a.getAttributeValue("id");
	    //add unique id to hashSet and update arcTable
	    idTable.put(id, label);
	    arcTable.put(id, label);
	    // namespace check?
	    add(label);
	    connectLabels(label);
	}

	instantiateSubnets();
	//System.out.println("DisplayPane:: loaded in labels");
    }

    public Clipboard getClipboard() {

	return myClipboard;
    }

    public void connectLabels(ArcLabel arclabel){
	NodeLabel source = (NodeLabel) getLabel(arclabel.getSourceID());
	NodeLabel target = (NodeLabel) getLabel(arclabel.getTargetID());
	arclabel.setSourceLabel(source);
	arclabel.setTargetLabel(target);
	target.addInputArc(arclabel);
	source.addOutputArc(arclabel);
    }

    public PNLabel getLabel(String id){
	Object o;
	
	if (transitionTable.containsKey(id))
	    o = transitionTable.get(id);
	else if (placeTable.containsKey(id))
	    o = placeTable.get(id);
	else if (arcTable.containsKey(id))
	    o = arcTable.get(id);
	else if (referencePlaceTable.containsKey(id))
	    o = referencePlaceTable.get(id);
	else if (subnetTable.containsKey(id))
	    o = subnetTable.get(id);
	else if (id.startsWith("_")) {
	    // id probably refers to an element within a subnet
	    int u = id.substring(1).indexOf('_') +1;
	    String subnetid = id.substring(1,u);
	    String elementid = id.substring(u+1);
	    System.out.println("Look in subnet "+subnetid);
	    
	    SubnetLabel l  = (SubnetLabel)subnetTable.get(subnetid);
	    if (l!=null) {
		o = l.instanceFrame.display.getLabel(elementid);
	    } else {
		System.out.println("DisplayPane::getLabel() - can't find label "+id+"!");
		o = null;
	    }
	} else if (id.indexOf('.')!=-1){
	    String subnet = "";
	    String pl = "";
	    int i = id.indexOf('.');
	    subnet = id.substring(0, i);
	    pl = id.substring(i+1);
	    // if the place is actually a place within a subnet, split the id as above
	    String place = "";
	    int j = pl.indexOf('.');
	    if (j != -1) {
		place = pl.substring(j+1);
	    } 
	    else place = pl;
	    if (subnetTable.containsKey(subnet)){
		SubnetLabel l  = (SubnetLabel)subnetTable.get(subnet);
		o = l.getImpExpLabel(place);
	    } else {
		System.out.println("DisplayPane::getLabel() - can't find label "+id+"!");
		o = null;
	    }
	}
	else {
	    System.out.println("DisplayPane::getLabel() - can't find label "+id+"!");
	    o = null;
	}
	return (PNLabel) o;
    }

    public void notify(PNLabel label) {
       
	String tool;

	// if we already have a source and target, clear them.
	if (source != null && target != null) {
	    source = null;
	    target = null;
	}
	
	// target is first click, source is second click.
	source = target;
	target = label;
	
	//if (source != null && target != null)
	//    System.out.println(source.getID() + " " + target.getID());
	

	// chosen tool - insert place, arc etc.
	tool = Application.setTool;
	
	if (tool.equals("arc") && source != null && target != null && source.myElementName() != target.myElementName())
	    newArc();
	else if (tool.equals("ref") && (source != null) && (target != null) && checkSourceTarget())
	    newRefLink();
    }

    private boolean checkSourceTarget() {

	String my_source = source.returnElement().getName();
	String my_target = target.returnElement().getName();

	if (my_source.equals("referencePlace") && my_target.equals("place"))
	    return true;
	else if (my_source.equals("importPlace") && my_target.equals("place"))
	    return true;
	else if (my_source.equals("referencePlace") && my_target.equals("exportPlace"))
	    return true;
	else if (my_source.equals("importPlace") && my_target.equals("exportPlace"))
	    return true;
	// else
	return false;
    }

    public void newPlace(int x, int y) {

	// Skips id's that are already in use.
	int placeID = 0;
	while (idTable.containsKey("p" + String.valueOf(placeID)))
	    placeID++;

	String unique_id = "p" + String.valueOf(placeID++); 
	Object[] arguments = {new Integer(x),new Integer(y), unique_id, ns};
	Element place = classman.newPlaceElement(arguments);
	PlaceLabel label = classman.newPlaceLabel(place);

	// add unique id to table and update placeTable.
	idTable.put(unique_id, label);
	placeTable.put(unique_id, label);

	// add new label to desktop and place to root element
	root.addContent(place);
	add(label);
    }

    public void newReferencePlace(int x, int y) {

	// Skips id's that are already in use.
	int referencePlaceID = 0;
	while (idTable.containsKey("r" + String.valueOf(referencePlaceID)))
	    referencePlaceID++;

	String unique_id = "r" + String.valueOf(referencePlaceID++); 
	Object[] arguments = {new Integer(x),new Integer(y), unique_id, ns};
	Element refPlace = classman.newReferencePlaceElement(arguments);
	ReferencePlaceLabel label = classman.newReferencePlaceLabel(refPlace);

	// add unique id to table and update placeTable.
	idTable.put(unique_id, label);
	referencePlaceTable.put(unique_id, label);

	// add new label to desktop and place to root element
	root.addContent(refPlace);
	add(label);
    }
        
    public void newTimedTransition(int x, int y) {

	TransitionLabel label = newTransition(x,y);
	try { 
	    TransitionStochasticLabel slabel = (TransitionStochasticLabel) label;
	    slabel.setTimed();
	} catch (Exception e) {}
	
	
    }

    public void newImmediateTransition(int x, int y) {

	TransitionLabel label = newTransition(x,y);
	try { 
	    TransitionStochasticLabel slabel = (TransitionStochasticLabel) label;
	    slabel.setImmediate();
	} catch (Exception e) {}
	
    }
    
    protected TransitionLabel newTransition(int x, int y) {

	// Skips id's that are already in use.
	int transitionID = 0;
	while (idTable.containsKey("t" + String.valueOf(transitionID)))
	    transitionID++;

	String unique_id = "t" + String.valueOf(transitionID++); 
	Object[] arguments = {new Integer(x),new Integer(y), unique_id, ns};
	Element transition = classman.newTransitionElement(arguments);
	TransitionLabel label = classman.newTransitionLabel(transition);

	// add unique id to table and update transitionTable.
	idTable.put(unique_id, label);
	transitionTable.put(unique_id, label);

	// add new label to desktop and transition to root element
	root.addContent(transition);
	add(label);
	
	return label;
    }
        
    public void newSubnet(int x, int y, String moduleRef, String importParam, String exportParam) {

	// Skips id's that are already in use.
	int subnetID = 0;
	while (idTable.containsKey("s" + String.valueOf(subnetID)))
	    subnetID++;

	String unique_id = "s" + String.valueOf(subnetID++); 
	Object[] arguments = {new Integer(x), new Integer(y), unique_id, moduleRef, importParam, exportParam, ns};
	Element instance = classman.newSubnetElement(arguments);
	SubnetLabel label = classman.newSubnetLabel(instance);

	// add unique id to table and update placeTable.
	idTable.put(unique_id, label);
	subnetTable.put(unique_id, label);

	// add new label to desktop and place to root element
	root.addContent(instance);
	add(label);
	repaint();
	label.createSubnetInstance();

    }

    public void newRefLink() {
	 
	// add reference link to JDOM
	Element sourceElement = source.returnElement();
	String targetID = target.getID();
	Element targetParentElement = target.returnElement().getParent();
	String targetParentID = targetParentElement.getAttributeValue("id");
	
	// if a reference link does not already exist
	if (sourceElement.getAttributeValue("ref") == null) {
	                 
	    // if the place is actually a place withing a subnet, split the id as above
	    String place = targetID;
	    int j = place.indexOf('.');
	    if (j != -1) {
		place = targetID.substring(j+1);
		//System.out.println("getLabel " + place);
	    } 

	    //System.out.println("new reflink");
	    sourceElement.setAttribute("ref", place);
	                    
	    // add new label to displayPane and source to refLinkTable
	    // (0 means added to front of container) 
	    RefLinkLabel label = new RefLinkLabel(source, target);
	    add(label,0);
	    repaint();
	
	    // add refNode target and sources.
	    NodeLabel my_source = (NodeLabel) source;
	    NodeLabel my_target = (NodeLabel) target;
	    my_source.addRefNodeTarget(my_target);
	    my_target.addRefNodeSource(my_source);
	                    
	    // add source and refLinkLabel to hashmap
	    refLinkTable.put(my_source, label);

	    // if reference is to a subnet.
	    if (targetParentElement.getName().equals("instance"))
		sourceElement.setAttribute("instance", targetParentID);
	}
    }
  
    public void newArc() {

	// Skips id's that are already in use.
	int arcID = 0;
	while (idTable.containsKey("a" + String.valueOf(arcID)))
	    arcID++;

	String unique_id = "a" + String.valueOf(arcID++); 

	int sourceX, sourceY, targetX, targetY, x, y;
	
	sourceX = source.getJX();
	sourceY = source.getJY();
	targetX = target.getJX();
	targetY = target.getJY();

	// calulate values for arc <position>.
	x = sourceX + (targetX - sourceX) / 2;
	y = sourceY + (targetY - sourceY) / 2;

	String sourceID = source.getID();
	String targetID = target.getID();

	// create arc.
	Object[] arguments = {new Integer(x),new Integer(y), unique_id, sourceID, targetID, ns};
	Element arc = classman.newArcElement(arguments);
	ArcLabel label = classman.newArcLabel(arc);

	// add unique id to table and update arcTable.
	idTable.put(unique_id, label);
	arcTable.put(unique_id, label);

	// add new label to desktop and arc to root element
	root.addContent(arc);
	add(label);

	// put the appropriate references in source and target nodes, and
	// in the arc itself
	connectLabels(label);

	// redraw DisplayPane with arc.
	repaint();
    }

    
    /** Pastes a copied placeLabel into DisplayPane @return new id of label. */
    public String pastePlace(Element clone) {

	String old_id = clone.getAttributeValue("id");
	String new_id = old_id;
	int placeNumber = 0;

	if (idTable.containsKey(old_id)) {

	    // Skips id's that are already in use.
	    while (idTable.containsKey("p" + String.valueOf(placeNumber)))
		placeNumber++;

	    new_id = "p" +  String.valueOf(placeNumber);
	}

	clone.setAttribute("id", new_id);
	PlaceLabel label = classman.newPlaceLabel(clone);

	// add unique id to table and update placeTable.
	idTable.put(new_id, label);
	placeTable.put(new_id, label);

	// add new label to desktop and place to root element
	root.addContent(clone);
	add(label);

	return new_id;
    }
    
    /** Pastes a copied Transitionlabel into DisplayPane @return new id of label. */
    public String pasteTransition(Element clone) {

	String old_id = clone.getAttributeValue("id");
	String new_id = old_id;
	int transitionNumber = 0;

	if (idTable.containsKey(old_id)) {

	    // Skips id's that are already in use.
	    while (idTable.containsKey("t" + String.valueOf(transitionNumber)))
		transitionNumber++;

	    new_id = "t" +  String.valueOf(transitionNumber);
	}

	clone.setAttribute("id", new_id);
	TransitionLabel label = classman.newTransitionLabel(clone);

	// add unique id to table and update placeTable.
	idTable.put(new_id, label);
	transitionTable.put(new_id, label);

	// add new label to desktop and place to root element
	root.addContent(clone);
	add(label);

	return new_id;
    }    
    
    /** Pastes a copied ReferencePlacelabel into DisplayPane @return new id of label. */
    public String pasteReferencePlace(Element clone) {

	String old_id = clone.getAttributeValue("id");
	String new_id = old_id;
	int refNumber = 0;

	if (idTable.containsKey(old_id)) {

	    // Skips id's that are already in use.
	    while (idTable.containsKey("r" + String.valueOf(refNumber)))
		refNumber++;

	    new_id = "r" +  String.valueOf(refNumber);
	}

	clone.setAttribute("id", new_id);
	ReferencePlaceLabel label = classman.newReferencePlaceLabel(clone);

	// add unique id to table and update placeTable.
	idTable.put(new_id, label);
	referencePlaceTable.put(new_id, label);

	// add new label to desktop and place to root element
	root.addContent(clone);
	add(label);

	return new_id;
    } 

     
    /** Pastes a copied Subnetlabel into DisplayPane @return new id of label. */
    public String pasteSubnetPlace(Element clone) {

	String old_id = clone.getAttributeValue("id");
	String new_id = old_id;
	int subNumber = 0;

	if (idTable.containsKey(old_id)) {

	    // Skips id's that are already in use.
	    while (idTable.containsKey("s" + String.valueOf(subNumber)))
		subNumber++;

	    new_id = "s" +  String.valueOf(subNumber);
	}

	clone.setAttribute("id", new_id);
	SubnetLabel label = classman.newSubnetLabel(clone);

	// add unique id to table and update placeTable.
	idTable.put(new_id, label);
	subnetTable.put(new_id, label);

	// add new label to desktop and place to root element
	root.addContent(clone);
	add(label);
	repaint();
	label.createSubnetInstance();

	return new_id;
    }

    /** Pastes a copied arclabel into DisplayPane @return new id of label. */
    public String pasteArcLabel(Element clone) {

	String old_id = clone.getAttributeValue("id");
	String new_id = old_id;
	int arcNumber = 0;
	String sourceID = source.getID();
	String targetID = target.getID();
	
	if (idTable.containsKey(old_id)) {

	    // Skips id's that are already in use.
	    while (idTable.containsKey("a" + String.valueOf(arcNumber)))
		arcNumber++;

	    new_id = "a" +  String.valueOf(arcNumber);
	}

	clone.setAttribute("id", new_id);
	clone.setAttribute("source", sourceID);
	clone.setAttribute("target", targetID);

	// create arcLabel.
	ArcLabel label = classman.newArcLabel(clone);

	// add unique id to table and update arcTable.
	idTable.put(new_id, label);
	arcTable.put(new_id, label);

	// add new label to desktop and arc to root element
	root.addContent(clone);
	add(label);

	// put the appropriate references in source and target nodes, and
	// in the arc itself
	connectLabels(label);

	// redraw DisplayPane with arc.
	repaint();

	return new_id;
    }


    // removes a PNLabel from the displayPane.
    public void removeLabel(PNLabel label) {

	boolean connected = false;
	java.util.List my_list = label.getLinkedLabels();

	// for all elements other than arcs
	if (!label.returnElement().getName().equals("arc")) {
	    if (my_list.size() > 0) {
		connected = true;
	    }
	}

	if (connected){
	    // remove connected arcs
	    Iterator it = my_list.iterator();
	    while (it.hasNext()) {
		PNLabel l = (PNLabel) it.next();
		removeMe(l);
	    }
	}
	// remove element itself.
	removeMe(label);
 
	repaint();
    }

    private void removeMe(PNLabel label) {

	// remove id from id tables.
	placeTable.remove(label.getID());
	referencePlaceTable.remove(label.getID());
	transitionTable.remove(label.getID());
	subnetTable.remove(label.getID());
	arcTable.remove(label.getID());
	idTable.remove(label.getID());
	

	// remove PNLabel from DisplayPane and from JDOM.
	label.remove();
    }
 
    public void removeSelectionBox() {
	if (mySelection!=null){
	    remove(mySelection);
	    mySelection = null;
	    repaint();
	}
    }

    public Dimension getPreferredSize() {
        return preferredSize;
    }

    /** Can be called to calculate the required size to 
     * display all the objects in the net.
     * The net must be already loaded in, as the positions of 
     * the PNLabels are examined to determine the size.
     * @see EditorFrame#EditorFrameListener
     */
    public void calcPreferredSize() {
	int numComponents = getComponentCount();
	Component[] comps = getComponents();
	
	int maxx=0, maxy=0, x=0, y=0;
	
	for (int i=0; i<numComponents; i++){
	    x = comps[i].getX();
	    y = comps[i].getY();
	    if (x>maxx) maxx = x;
	    if (y>maxy) maxy = y;
	}
	if (x>0 && y>0)
	    // allow some extra for width/height of labels on screen
	    preferredSize.setSize(maxx+80,maxy+80);
	else
	    preferredSize.setSize(800,800);
    }

    // returns true if the root element is module.
    public boolean isModule() {

	return this.root.getName().equals("module");
    }

    public void paintComponent(Graphics g){
        super.paintComponent(g);
	paintComponents(g);

	if (isModule()) {
	    Graphics2D interfaceLine = (Graphics2D)g;
	    interfaceLine.setStroke(new BasicStroke(2.0f));
	    interfaceLine.draw(new Line2D.Double(0,75,1000,75));
	}
    }

    // add newElements to the display 
    // Element already exists but creating new element deals with issue of id.
    public void addNode(Element e) {
	
	// for all elements except subnet instances, remove attribute for reference arc.
	if (!e.getName().equals("instance"))
	    e.removeAttribute("ref");

	String old_id = e.getAttributeValue("id");
	String new_id = "";
	
	int x = getJX(e);
	int y = getJY(e);

	if (e.getName().equals("place")) {
	    new_id = pastePlace(e); 
	}
	else if (e.getName().equals("transition")) {
	    new_id = pasteTransition(e);
	}
	else if (e.getName().equals("referencePlace")) {
	    new_id = pasteReferencePlace(e);
	}
	else if (e.getName().equals("instance")) {
	    new_id = pasteSubnetPlace(e);
	}
	else // element not found
	    return;
	
	idmapping.put(old_id, new_id);

    }
    
    /** Pastes a copied arc into DisplayPane */
    public void addArc(Element e){

	String old_id = e.getAttributeValue("id");
	String new_id = "";

	if (e.getName().equals("arc")) {
	    setSourceTarget(e);
	    new_id = pasteArcLabel(e); 
	}

	idmapping.put(old_id, new_id);
    }


    // get X, Y values from the JDOM Document
    private int getJX(Element e) {
	return Integer.parseInt(e.getChild("graphics",ns).getChild("position",ns).getAttributeValue("x"));
    }
    private int getJY(Element e) {
	return Integer.parseInt(e.getChild("graphics",ns).getChild("position",ns).getAttributeValue("y"));
    }

    private String getSource(Element e) {
	return e.getAttributeValue("source");
    }

    private String getTarget(Element e) {
	return e.getAttributeValue("target");
    }

    // set source and target PNLabels.
    private void setSourceTarget(Element e) {

	String old_source = getSource(e);
	String old_target = getTarget(e);
	String new_source = (String) idmapping.get(old_source);
	String new_target = (String) idmapping.get(old_target); 

	// find labels in idTable
	source = (PNLabel) idTable.get(new_source);
	target = (PNLabel) idTable.get(new_target);

    }

    /** Brings up the right-click popup menu.
     *  The class is determined by the local variable propertiesbox. */
    public void showContextMenu(MouseEvent e) {
	try {
	    PetriContextMenu popup; 
	    Class[] argtypes = new Class[] {DisplayPane.class,Class.class};
	    Constructor constructor = contextmenu.getConstructor(argtypes);
	    Object[] args = new Object[] {this, propertiesbox};
	    popup = (PetriContextMenu) constructor.newInstance(args);
	    popup.show(this, e.getX(), e.getY());
	    //System.out.println("DisplayPane::Successfully created context menu");
	} catch (Exception ex) {
	    System.out.println("DisplayPane::Error creating context menu!");
	}
    }

    protected void instantiateSubnets() {
	//System.out.println("DisplayPane:: instantiating subnets");
	Iterator subs = subnetTable.values().iterator();
	while (subs.hasNext()) {
	    SubnetLabel sub = (SubnetLabel) subs.next();
	    sub.createSubnetInstance();
	}
    }
    
    public void closeSubnetInstances() {
	//System.out.println("DisplayPane:: closing subnet instances");
	Iterator subs = subnetTable.values().iterator();
	while (subs.hasNext()) {
	    SubnetLabel sub = (SubnetLabel) subs.next();
	    sub.closeSubnetInstance();
	}
    }
        
    public void linkSubnet(SubnetImportLabel importLabel, SubnetExportLabel exportLabel){
	// import and export are labels in a subnet instance in another
	// <net> or <module> than the one shown in this window.
	
	// deal with the import place
	// 1. find the reference place in this subnet that is linked
	//    to the import place
	//    * assumes there is only one such reference place
	//    * it may also be an import place in another contained subnet!
	//System.out.println("Dealing with import place");
	String id = importLabel.getID();
	//System.out.println("- find reference place with id "+id);
	ReferencePlaceLabel refplace = null;
	if (referencePlaceTable.containsKey(id))
	    refplace = (ReferencePlaceLabel) referencePlaceTable.get(id);
	else ; // check for import places in subnets

	// 2. link the reference place to the import place in the <instance>,
	//    and v.v.
	if (refplace!=null) {
	    refplace.addRefNodeTarget(importLabel);
	    importLabel.addRefNodeSource(refplace);
	    //System.out.println("Set targets");
	}
	
	// deal with the export place
	//System.out.println("Dealing with export place");
	id = exportLabel.getID();
	//System.out.println("- find reference place with id "+id);
	refplace = null;
	if (referencePlaceTable.containsKey(id))
	    refplace = (ReferencePlaceLabel) referencePlaceTable.get(id);
	else ; // check for export places in subnets

	// 2. link the reference place to the export place in the <instance>,
	//    and v.v.
	if (refplace!=null) {
	    refplace.addRefNodeSource(exportLabel);
	    exportLabel.addRefNodeTarget(refplace);
	    //System.out.println("Set targets");
	}
    }
    
    public void highlightEnabled(boolean highlight){
	if (eframe.subnet==null) return;
	eframe.subnet.highlightEnabled(highlight);
	eframe.subnet.repaint();
    }
    
    public void highlightFire(){
	if (eframe.subnet==null) return;
	eframe.subnet.highlightFire();
	eframe.subnet.repaint();
    }    
}


















