/*Listener for DisplayPane mouse events
*/

package uk.ac.ic.doc.y2002.tool.application;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import org.jdom.*;
import uk.ac.ic.doc.y2002.tool.editor.*;
import java.util.*;

class DisplayPaneListener extends MouseInputAdapter{

    private boolean beingDrawn = false;
    private SelectionBox my_selection = null; 
    private int cornerX, cornerY, width, height;

    // empty constructor
    public DisplayPaneListener() {}

    // when the mouse is pressed checks to see which tool is selected
    // and takes appropriate action.
    public void mousePressed(MouseEvent e) {

	int x = e.getX();
	int y = e.getY();

	DisplayPane display = (DisplayPane) e.getSource();
	String tool = Application.setTool;
	
	if ( (e.getModifiers() & InputEvent.BUTTON3_MASK) == InputEvent.BUTTON3_MASK) {
	    // right button clicked
	    // opens right-click context menu
	    display.showContextMenu(e);
	} else if (tool.equals("place"))
	    display.newPlace(x,y);
	else if (tool.equals("immediate"))
	    display.newImmediateTransition(x,y);
	else if (tool.equals("timed"))
	    display.newTimedTransition(x,y);
	else if (tool.equals("subnet")){
	    display.newSubnet(x, y, SubnetLabel.chosenRef(), SubnetLabel.importPlace(), SubnetLabel.exportPlace());
	    
	    /*Element temp = (Element)  (SubnetLabel.chosenRoot).clone();
	    java.util.List subnets = display.eframe.doc.getRootElement().getChildren("module", display.eframe.ns);
	    Iterator is = subnets.iterator();
	    boolean isSame = false; 
	    while (is.hasNext()) {
		is.next();
		String moduleName = display.eframe.doc.getRootElement().getAttributeValue("name");
		if (SubnetLabel.chosenRef.equals(moduleName)) isSame = true;
	    }
	    if (!isSame)
	    display.eframe.doc.getRootElement().addContent(temp);*/
	}
	else if (tool.equals("refPlace"))
	    display.newReferencePlace(x,y);
	else if (tool.equals("pointer")) {
	    beingDrawn = true;
	    cornerX = e.getX();
	    cornerY = e.getY();
	    drawBox(e);
	}

	display.repaint();
    }

    public void mouseReleased(MouseEvent e) {
	beingDrawn = false;
	String tool = Application.setTool;
	
	if (tool.equals("pointer") && my_selection!=null) {
	    // calculate selection of contained labels
	    DisplayPane display = (DisplayPane) e.getSource();
	    my_selection.calculate(display.idTable);
	}
    }

    // function to create selectionBox
    private void drawBox(MouseEvent e) {
	 
	DisplayPane display = (DisplayPane) e.getSource();
  
	// if a SelectionBox already exists
	if (my_selection != null) {
	    display.remove(my_selection);
	}

	my_selection = new SelectionBox(cornerX, cornerY);
	display.mySelection = my_selection;
	display.add(my_selection);
    }
    
    // Handles the event of a user dragging the mouse while holding
    // down the mouse button.
    public void mouseDragged(MouseEvent e){
	if ( beingDrawn) {
	    updateLocation(e);
	}
	    
    }
    
    // in response to a Mouse event
    protected void updateLocation(MouseEvent e){
	
	DisplayPane display = (DisplayPane) e.getSource();
	int newX, newY;
	newX = e.getX();
	newY = e.getY();
	
	width = newX - cornerX;
	height = newY - cornerY;

	// stretch SelectionBox
	my_selection.stretchBox(width, height);

	display.repaint();
    }
   
}









