package uk.ac.ic.doc.y2002.tool.application;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import org.jdom.*;
import uk.ac.ic.doc.y2002.tool.editor.*;

public class EditorFrame extends JInternalFrame {
    Document doc; // the whole JDOM Document
    public Element root; // the <net> or <module> this frame is working on
    Namespace ns; // the namespace of the 'root' element
    public File file; 
    public Application app;
    static int openFrameCount = 0;
    static final int xOffset = 30, yOffset = 30;
    public DisplayPane display;
    public SubnetLabel subnet;

    public EditorFrame(Application app, Document doc, Element root, File file) {
        super("Petri Net " + (++openFrameCount), 
              true, //resizable
              true, //closable
              true, //maximizable
              true);//iconifiable

	this.app = app;
	// load in the document
	loadDocument(doc, root, file);
	//System.out.println("EditorFrame::loaded document");

        //...Create the GUI and put it in the window...
	display = new DisplayPane(this, root);
	Container contentPane = getContentPane();
	JScrollPane scrollPane = new JScrollPane(display);
	contentPane.add(scrollPane);
	//System.out.println("EditorFrame::created DisplayPane");

        //...Then set the window size or call pack...
        setSize(500,500);

        //Set the window's location.
        setLocation(xOffset*openFrameCount, yOffset*openFrameCount);

	// Set the window's title (from the name of the net, if given,
	// or from the filename)
	generateTitle();

	// add window listener
	EditorFrameListener listener = new EditorFrameListener();
	addInternalFrameListener(listener);
	addComponentListener(listener);
	setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	
    }
    
    // initialisation
    public void loadDocument(Document doc, Element root, File file) {
	this.doc = doc;
	this.root = root;
	ns = root.getNamespace();
	this.file = file;
    }

    public void generateTitle() {
	// Set the window's title, from the name of the Petri net loaded
	String title = root.getChild("name",ns).getChild("value",ns).getText();
	if (title.length()>0)
	    setTitle("Petri Net: "+title);
	else 
	    setTitle("Petri Net <"+file.toString()+">");
    }

    public boolean inSubnet() {
	if (subnet!=null) return true;
	return false;
    }
    
    // internal classes
    /** A window listener */
    
    protected class EditorFrameListener extends InternalFrameAdapter implements InternalFrameListener, ComponentListener{
	
	public EditorFrameListener(){
	    super();
	}

	public void internalFrameClosing(InternalFrameEvent e){
	    //System.out.println("EditorFrameListener:: Internal frame closing");
	    EditorFrame f = (EditorFrame) e.getSource();
	    f.display.animator.pause();

	    if (display.updated) {
		int response = JOptionPane.showConfirmDialog(display, new String[] {
		    "This net has not been saved.",
		    "Save before closing?"}, "Save before closing?", 
		    JOptionPane.YES_NO_CANCEL_OPTION,
		    JOptionPane.WARNING_MESSAGE);

		switch(response) {
		case JOptionPane.YES_OPTION: {
		    // call save
		    f.app.simulateSave();  
		    dispose();
		          
		}
		case JOptionPane.NO_OPTION: if (inSubnet()) 
		    setVisible(false); // hide frame
		else dispose(); // quit don't save
		case JOptionPane.CANCEL_OPTION: // don't quit
		case JOptionPane.CLOSED_OPTION: // don't quit
		}
	    }
	         
	}
	
	public void internalFrameActivated(InternalFrameEvent e){
	    //System.out.println("EditorFrameListener:: Internal frame activated");
	    EditorFrame f = (EditorFrame) e.getSource();
	    f.app.setAnimate(inSubnet());
	}
	
	public void componentHidden(ComponentEvent e) {}

	public void componentMoved(ComponentEvent e) {}
	
	public void componentResized(ComponentEvent e) {
	    //System.out.println("EditorFrameListener:: Internal frame resized");
	    EditorFrame f = (EditorFrame) e.getSource();
	    f.display.calcPreferredSize();
	}

	public void componentShown(ComponentEvent e) {}
	
    }
    
   

}


















