package uk.ac.ic.doc.y2002.tool.application;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import org.jdom.*;
import uk.ac.ic.doc.y2002.tool.editor.*;

public class GSPNAnimator extends PNAnimator {

    public GSPNAnimator(DisplayPane display) {
	super(display);
	System.out.println("GSPNAnimator:: initialised");
    }

    /** Highlights enabled transitions and returns a List of enabled transitions 
     * @return List of enabled transitions (the TransitionLabels) 
     */
    public java.util.List highlightEnabledTransitions(boolean highlight){
	java.util.List trans = new java.util.LinkedList();
	HashMap m = display.transitionTable;
	boolean foundImmediate=false;

	for (Iterator i=m.entrySet().iterator(); i.hasNext();) {
	    Map.Entry e = (Map.Entry) i.next();
	    TransitionStochasticLabel l = (TransitionStochasticLabel) e.getValue();
	    if (l.isActive()){
		l.highlightEnabled(highlight);
		trans.add(l);
		if (l.isImmediate())
		    foundImmediate = true;
	    } else {
		l.highlightEnabled(!highlight);
	    }
	    if (foundImmediate){
		// remove all timed transitions from the active list
		// and un-highlight them
		java.util.List trans2 = new java.util.LinkedList();
		Iterator ts= trans.iterator();
		while (ts.hasNext()) {
		    TransitionStochasticLabel t = (TransitionStochasticLabel) ts.next();
		    if (t.isImmediate()){
			trans2.add(t); // add immediate transitions to the new list
		    } else {
			// un-highlight timed transitions
			t.highlightEnabled(false);    
		    }
		}
		trans = trans2;
	    }
	}
	return trans;
    }

    protected TransitionLabel chooseFire(java.util.List trans){
	// determines which transitions have the highest weight
	// chooses a random transition from these
	if (trans.size()>0){
	    TransitionStochasticLabel t = (TransitionStochasticLabel) trans.get(0);
	    TransitionLabel l=null;

	    // For both immediate and stochastic transitions, use
	    // probability of a transition firing = transition weight/sum of all weights

	    // the list contains all eligible transitions
	    // search through them to determine the total weights
	    float weights = 0;
	    Iterator ts= trans.iterator();
	    while (ts.hasNext()) {
		t = (TransitionStochasticLabel) ts.next();
		weights += t.getWeight();
	    }
	    // get random float between 0 - 1.0
	    // - uniformly distributed
	    float rand = new Random().nextFloat();
	    // bring it into the right range
	    rand *= weights;

	    // go through the transitions again, stopping at the
	    // one whose range holds the random number
	    Iterator tts= trans.iterator();
	    float minweight=0, maxweight=0;
	    while (tts.hasNext()) {
		t = (TransitionStochasticLabel) tts.next();
		maxweight = minweight + t.getWeight();
		if (rand >=minweight && rand < maxweight){
		    l = t;
		    break;
		}
		minweight = maxweight;
	    }
	    
	    System.out.println("GSPNAnimator:: firing transition "+l.getID());
	    return l;
	} else {
	    System.out.println("GSPNAnimator:: no transition enabled to fire!");
	    return null;
	}
    
    }
    
}    
    
    
    
    
