/*	Listener for Edit Menu Cut
*/

package uk.ac.ic.doc.y2002.tool.application;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import org.jdom.*;
import java.util.*;
import uk.ac.ic.doc.y2002.tool.editor.*;

class MenuEditCutListener implements ActionListener{

    private JDesktopPane desktop; // used to locate current desktop.
    private SelectionBox clip; // list of cut or copied elements. 

    public MenuEditCutListener(JDesktopPane desktop) {

	this.desktop = desktop;
    }

    public void actionPerformed(ActionEvent e) {

	EditorFrame my_frame = (EditorFrame) desktop.getSelectedFrame();       
	MyMenuItem item = (MyMenuItem) e.getSource();
	String cmd = item.getActionCommand();
	DisplayPane display = my_frame.display;
	clip = display.mySelection;

	// if we have a selected editor frame.
	if (my_frame != null) {
	    if (cmd.equals("cut") && clip != null)
		getLabels(my_frame);
	    else if (cmd.equals("delete"))
		deleteLabels(my_frame);
	}
    }
 
    // deletes labels.
    private void deleteLabels(EditorFrame frame) {
	
	DisplayPane display = frame.display;
	Clipboard myClipboard = display.getClipboard();

	// make a copy of hashmap to avoid concurrent access problems - remove arcs.
	HashMap m = new HashMap(display.idTable);
	Iterator it = m.entrySet().iterator();
      
	while (it.hasNext() ) {

	    Map.Entry e = (Map.Entry) it.next();
	    PNLabel label = (PNLabel) e.getValue();
	    
	    // if PNLabel is a selected arc
	    if (label.returnSelected() && label.myElementName().equals("arc"))
		cutLabel(display, label);
	    
	}

	// make a copy of new hasmap , with arcs removed. Cut nodes and remaining attached arcs.
	HashMap m2 = new HashMap(display.idTable);
	Iterator it2 = m2.entrySet().iterator();
      
	while (it2.hasNext() ) {

	    Map.Entry e = (Map.Entry) it2.next();
	    PNLabel label2 = (PNLabel) e.getValue();

	    // if PNLabel selected.
	    if (label2.returnSelected()) 
		    cutLabel(display, label2);
	}

	display.removeSelectionBox();
	
	// remove links to source and target labels (for arc creation)
	display.source = null;
	display.target = null;
    }


    private void getLabels(EditorFrame frame) {

	DisplayPane display = frame.display;
	Clipboard myClipboard = display.getClipboard();

	// make a copy of hashmap to avoid concurrent access problems - remove arcs.
	HashMap m = new HashMap(display.idTable);
	Iterator it = m.entrySet().iterator();
      
	while (it.hasNext() ) {

	    Map.Entry e = (Map.Entry) it.next();
	    PNLabel label = (PNLabel) e.getValue();
	    
	    // if PNLabel is a selected arc
	    if (label.returnSelected() && label.myElementName().equals("arc"))
		cutLabel(display, label);
	    
	}

	// make a copy of new hasmap , with arcs removed. Cut nodes and remaining attached arcs.
	HashMap m2 = new HashMap(display.idTable);
	Iterator it2 = m2.entrySet().iterator();
      
	while (it2.hasNext() ) {

	    Map.Entry e = (Map.Entry) it2.next();
	    PNLabel label2 = (PNLabel) e.getValue();

	    // if PNLabel selected.
	    if (label2.returnSelected()) 
		    cutLabel(display, label2);
	}

	// Add clip to clipboard if not empty clip.
	if (clip != null  && clip.getContainedElements().size() > 0) {
	    clip.setCutOffset();
	    myClipboard.addClip(clip);
	}

	display.removeSelectionBox();
	
	// remove links to source and target labels (for arc creation)
	display.source = null;
	display.target = null;
    } 

    private void cutLabel(DisplayPane display, PNLabel label) {
	
	// ask display pane to remove PNLabel and selection box.
	display.removeLabel(label);
       
        // remove remaining attached arcs
	java.util.List attached = label.getLinkedLabels();
	Iterator it = attached.iterator();

	while (it.hasNext()) {
	    
	    PNLabel cut = (PNLabel) it.next();
	    if (!cut.returnSelected()) {
		System.out.println("attached - not selected");
	    }
	}
  
    }

}









