/*  dynamic load module (reflection)
*/
package uk.ac.ic.doc.y2002.tool.application;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.util.*;
import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;
import java.lang.reflect.*;
import java.lang.*;

public class MenuLoadModuleListener implements ActionListener {
  Application adaptee;
  JFileChooser jFileChooser = new JFileChooser();
  String CLASSPATH = "./modules";

  public MenuLoadModuleListener(Application adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {

      String myClassName = null;    // full path with filename
      String trimmed =null;         // myClassName without .class
      String moduleName = null;     // the return module discription
      Object object = null;         // the current Object

      // chose a class file and make the object.
      jFileChooser.setCurrentDirectory(new File(CLASSPATH));
      ExampleFileFilter filter = new ExampleFileFilter();
      filter.addExtension("class");
      filter.setDescription("Java Class Files");
      jFileChooser.addChoosableFileFilter(filter);
      // test return for Approve / Cancel
      if (JFileChooser.APPROVE_OPTION == jFileChooser.showOpenDialog(adaptee)) {
        //set the current file name to the user's seletion,
        myClassName = jFileChooser.getSelectedFile().getName();
        System.out.println("MenuLoadModuleListener:: Loading Module:" + jFileChooser.getSelectedFile().getPath());
        int dotPos = myClassName.lastIndexOf(".");
       try {
           trimmed = new String(myClassName.toCharArray(), 0, dotPos);
        }catch(StringIndexOutOfBoundsException t){
          System.out.print("MenuLoadModuleListener:: Error trimming the class Name" + myClassName);
	  loaded(false);
          return;
        }
        object = (Object) createObject(trimmed);
        adaptee.repaint();
      }
      else {
        adaptee.repaint();
      }


      if (object == null){
        System.out.println("MenuLoadModuleListener:: Error creating Module");
	loaded(false);
        return;
      }

      // for submenu.
      int position = -1;
      for(int i = 0; i < adaptee.runModule.length ; ++i) {
        if (adaptee.module[i] == null) {
          adaptee.module[i] = object;
          position = i;
          break;   // break for loop
        }
      }

      // invoke getModuleName method and get name to moduleName.
      Class[] parameterTypes = new Class[] {};
      Method runModuleMethod;
      Object[] arguments = new Object[] {};
      Class moduleClass = object.getClass();
      try {
        runModuleMethod = moduleClass.getMethod("getModuleName", parameterTypes);
        moduleName = (String)runModuleMethod.invoke(object, arguments);
      } catch (NoSuchMethodException t) {
          System.out.println("MenuLoadModuleListener:: Error loading module!");
	  loaded(false);
      } catch (IllegalAccessException t) {
          System.out.println("MenuLoadModuleListener:: Error invoking method!");
	  loaded(false);
      } catch (InvocationTargetException t) {
          t.printStackTrace();
      }

      //System.out.println("ok, GET THE OBJECT:");

      if (position != -1) {
        adaptee.runModule[position].setText(moduleName);
        adaptee.runModule[position].setVisible(true);
        adaptee.removeModule[position].setText(moduleName);
        adaptee.removeModule[position].setVisible(true);
      } else {
        System.out.println("MenuLoadModuleListener:: too many modules loaded, remove a module then insert again");
	JOptionPane.showMessageDialog(adaptee,"Too many modules loaded - please unload one and try again","Too many modules!",JOptionPane.PLAIN_MESSAGE);
        return;
     }
      loaded(true);
  }

  private Object createObject(String className) {
    Object object = null;
    try {
        Class classDefinition = Class.forName(className);
        object = classDefinition.newInstance();
    } catch (InstantiationException e) {
        System.out.println("MenuLoadModuleListener:: Error loading Module");
        //System.out.println(e);
    } catch (IllegalAccessException e) {
        System.out.println("MenuLoadModuleListener:: Class could not be accessed");
        //System.out.println(e);
    } catch (ClassNotFoundException e) {
        //System.out.println(e);
       System.out.println("MenuLoadModuleListener:: Class could not be found");
    } catch (NoClassDefFoundError e) {
	System.out.println("MenuLoadModuleListener:: Class could not be found");
    }
    return object;
  }

  public void runAModule(){
    Object[] arguments;
    EditorFrame my_frame = (EditorFrame) adaptee.desktop.getSelectedFrame();
    Document doc = my_frame.doc;
    ModuleBridge bridge = new ModuleBridge(my_frame.display);
    // flat modules
    if (hasSubnet(doc) && !checkIsSubnetable() ) {
      Document flatDoc = flatModule(doc);
      arguments = new Object[] {flatDoc, bridge};
      System.out.println("flat all the modules(subnets)");
    } else {
      System.out.println("do not need flat the modules");
      arguments = new Object[] {doc, bridge};
    }

    //now use Reflection methods to obtain the runModule method and invoke it
    Class[] parameterTypes = new Class[] {Document.class, ModuleBridge.class};
    Method runModuleMethod;
    // Object[] arguments = new Object[] {doc};
    Class moduleClass = Application.currentModule.getClass();
    try {
      runModuleMethod = moduleClass.getMethod("runModule", parameterTypes);
      runModuleMethod.invoke(Application.currentModule, arguments);
    } catch (NoSuchMethodException t) {
        System.out.println("MenuLoadModuleListener:: Error loading module!");
	loaded(false);
    } catch (IllegalAccessException t) {
        System.out.println("MenuLoadModuleListener:: Error invoking method!");
	loaded(false);
    } catch (InvocationTargetException t) {
        System.out.println(t);
	loaded(false);
        //t.printStackTrace();
    }
  }

  public boolean checkIsSubnetable() {
      Class[] parameterTypes = new Class[] {};
      Method runModuleMethod;
      Object[] arguments = new Object[] {};
      Class moduleClass = Application.currentModule.getClass();
      String temp = new String();
      try {
        runModuleMethod = moduleClass.getMethod("isSubnetable", parameterTypes);
        temp = (String) runModuleMethod.invoke(Application.currentModule, arguments);
	System.out.println(temp);
      } catch (NoSuchMethodException t) {
          System.out.println("MenuLoadModuleListener:: Error loading module!!!!!!");
	  loaded(false);
      } catch (IllegalAccessException t) {
          System.out.println("MenuLoadModuleListener:: Error invoking method!");
	  loaded(false);
      } catch (InvocationTargetException t) {
          t.printStackTrace();
	  loaded(false);
      }
      return ((temp.equalsIgnoreCase("true")) ? true : false) ;
  }

  private boolean hasSubnet(Document doc) {
      Element root = doc.getRootElement();
      Namespace ns = root.getNamespace();
      Element net = root.getChild("net",ns);
      // two list for flatting
      // java.util.List subnets = root.getChildren("module",ns);
      java.util.List instances = net.getChildren("instance",ns);
      return (instances.size() == 0) ? false : true;
  }

      private String getFilePath(Element element) {
	String p = element.getAttributeValue("ref");
	System.out.println(p);
	int end = p.indexOf('#');
	System.out.println(end);
	String path = p.substring(0, end);
	System.out.println(path);
	return path;
        }


  private Document flatModule(Document doc){
      Document flatDoc = (Document) doc.clone();
      Element root = flatDoc.getRootElement();
      Namespace ns = root.getNamespace();
      Element net = root.getChild("net",ns);
      // two list for flatting
      // java.util.List subnets = root.getChildren("module",ns);
      java.util.List instances = net.getChildren("instance",ns);
      Iterator is = instances.iterator();
      while (is.hasNext()) {
            // instance
            Element instance = (Element) is.next();
            String subnetID = instance.getAttributeValue("id");
            String subnetRef = instance.getAttributeValue("ref");
            Element importPlace = (Element) instance.getChild("importPlace",ns);
            String parameter;
            String placeRef;
            parameter = importPlace.getAttributeValue("parameter");
            if (importPlace.getAttributeValue("instance")!=null)
                    placeRef = "_" + importPlace.getAttributeValue("instance") + "_" +
                              importPlace.getAttributeValue("ref");
            else
                    placeRef = importPlace.getAttributeValue("ref");
            // find corresponing subnet and add new Place, t , arc, to net

            File myFile = new File(this.getFilePath(instance));
	    // Request document building without validation.
	    SAXBuilder builder = new SAXBuilder(false);
            Document doctemp = null;
	    try {
	        doctemp = builder.build(myFile);
		}
            catch (Exception error) { System.out.println("Error - file not read."); }

            Element roottemp = doctemp.getRootElement();
            Namespace nstemp = roottemp.getNamespace();
            Element subnet = roottemp.getChild("module", nstemp);

            java.util.List subPlaces = subnet.getChildren("place",ns);
            java.util.List subTransitions = subnet.getChildren("transition",ns);
            java.util.List subArcs = subnet.getChildren("arc",ns);
            // add tansitions
            Iterator i = subTransitions.iterator();
            while (i.hasNext()) {
              Element temp = (Element) i.next();
              Element element = (Element) temp.clone();
              String id = element.getAttributeValue("id");
              String idNew = "_" + subnetID + "_" + id;
              element.setAttribute("id", idNew);
              String name = element.getChild("name",ns).getChildText("value",ns);
              String nameNew = "_" + subnetID + "_" + name;
              element.getChild("name",ns).getChild("value",ns).setText(nameNew);
              net.addContent(element);
              System.out.println("* just added a Transion to net ");
            }   // end of while(i.hasNext)

            // add Places
            i = subPlaces.iterator();
            while (i.hasNext()) {
              Element temp = (Element) i.next();
              Element element = (Element) temp.clone();
              String id = element.getAttributeValue("id");
              String idNew = new String();
              String name = element.getChild("name",ns).getChildText("value",ns);
              String nameNew = "_" + subnetID + "_" + name;
              element.getChild("name",ns).getChild("value",ns).setText(nameNew);
              if ( id.equalsIgnoreCase("y"))
                 idNew = "_" + subnetID + "_" + "p2";
              else
                 idNew =  "_" + subnetID + "_" + id;
              element.setAttribute("id", idNew);
              net.addContent(element);
              System.out.println("* just added a Place to net ");
            }   // end of while(i.hasNext)

            // add Arcs
            i = subArcs.iterator();
            while (i.hasNext()) {
              Element temp = (Element) i.next();
              Element element = (Element) temp.clone();

              String id = element.getAttributeValue("id");
              id = "_" + subnetID + "_" + id;
              element.setAttribute("id", id);

              id = element.getAttributeValue("source");
              if ( id.equalsIgnoreCase("y"))
                 id = "_" + subnetID + "_" + "p2";
              else if ( id.equalsIgnoreCase("x"))
                 id = placeRef;
              else
                 id =  "_" + subnetID + "_" + id;
              element.setAttribute("source", id);

              id = element.getAttributeValue("target");
              if ( id.equalsIgnoreCase("y"))
                 id = "_" + subnetID + "_" + "p2";
              else if ( id.equalsIgnoreCase("x"))
                 id = placeRef;
              else
                 id =  "_" + subnetID + "_" + id;
              element.setAttribute("target", id);

              net.addContent(element);
              System.out.println("* just added a Arc to net ");
            }   // end of while(i.hasNext)
    } // end while(is.hasNext())
    //root.removeChildren("module",ns);
    net.removeChildren("instance",ns);
    return flatDoc;
  }

    private void loaded(boolean load){
	if (!load)
	    //System.out.println("MenuLoadModuleListener:: failed to load module");
	    JOptionPane.showMessageDialog(adaptee,"Error loading module!","Error loading module",JOptionPane.PLAIN_MESSAGE);

	else
	    //System.out.println("MenuLoadModuleListener:: module loaded successfully");
	    JOptionPane.showMessageDialog(adaptee,"Loaded module successfully - run it from the Modules menu.","Module Loaded",JOptionPane.PLAIN_MESSAGE);

    }

}
















