/*    Listener for File Menu - New
*/

package uk.ac.ic.doc.y2002.tool.application;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.util.*;
import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;

class MenuNewListener implements ActionListener{

    private JDesktopPane desktop; // used to locate current desktop.
    private Application appframe;


    public MenuNewListener(Application appframe, JDesktopPane desktop) {
	this.appframe = appframe;
	this.desktop = desktop;
    }


    public void actionPerformed(ActionEvent e) {
	String cmd = e.getActionCommand();
	ArrayList types = new ArrayList();
	ArrayList ids = new ArrayList();
	ArrayList templates = new ArrayList();

	// get the list of net types from nettypes.xml
	// Request document building without validation
	SAXBuilder builder = new SAXBuilder(false);
	try {
	    Document doc = builder.build(new File("nettypes.xml"));
	    // find the given nettype
	    Element root = doc.getRootElement(); // <pnmlnettypes>
	    getTypes(root, types, ids, templates);
	} catch (Exception ex) {
	    System.out.println("MenuNewListener:: Error reading nettypes.xml - "+ex);
	}

	if (types.isEmpty()){
	    types.add("Place/Transition Net");
	    ids.add("ptNet");
	    templates.add("./templates/template_ptnet.xml");
	}


	// create net type chooser and get input
	Object input;
	String title = "";
	if (cmd.equals("newnet"))
	    title = "New net - choose net type";
	else
	    title = "New subnet - choose net type";
	input = JOptionPane.showInternalInputDialog(desktop,"Choose a petri net type",title,
						    JOptionPane.QUESTION_MESSAGE,null,types.toArray(),types.get(0));
	//System.out.println("MenuNewListener:: User selected "+input);
	if (input==null) {
	    // user selected Cancel
	    return;
	}

	// find the selected option
	int index = types.indexOf(input);
	//System.out.println("MenuNewListener:: User selected "+ids.get(index)+" "+templates.get(index));

	// determine whether a Net or Subnet was requested
	File myFile;
	if (cmd.equals("newnet")){
	    // new Net
	    System.out.println("MenuNewListener:: making new net");
	    // load in net template
	    myFile = new File((String) templates.get(index));
	} else {
	    // new Subnet
	    System.out.println("MenuNewListener:: making new subnet");
	    myFile = new File("./templates/template_subnet.xml");
	}

	// to avoid template overwriting.
	File newFile = new File("./new.xml");

	try {
	    Document doc = builder.build(myFile);
            examineDoc(doc, newFile, (String)ids.get(index));
	} catch (Exception error) { System.out.print("Error opening file"); }

    }

    // Method to examine loaded JDOM document.
    private void examineDoc(Document doc, File newFile, String nettype) {

	Element root = doc.getRootElement();
	Namespace ns = root.getNamespace();

	EditorFrame frame;
	Element net_content;

	if (root.getChildren("net",ns).size()>0){
	    // got a <net>
	    net_content = root.getChild("net", ns);
	    frame = new EditorFrame(appframe, doc, net_content, newFile);
	} else  if (root.getChildren("module",ns).size()>0){
	    // got a <module>  (subnet)
	    net_content = root.getChild("module",ns);
	    net_content.setAttribute("type",nettype);
	    frame = new SubnetFrame(appframe, doc, net_content, newFile);
	} else {
	    System.out.println("MenuNewListener:: error opening new net");
	    return;
	}
	frame.setVisible(true);
	desktop.add(frame);
	try {
	    frame.setSelected(true);
	} catch (java.beans.PropertyVetoException e) {};
	appframe.setAnimate(frame.inSubnet());
    }

    // Method to get the net types from file
    private void getTypes(Element e, ArrayList types, ArrayList ids, ArrayList templates){
	java.util.List nettypes = e.getChildren("nettype");
	Iterator ts= nettypes.iterator();
	while (ts.hasNext()) {
	    Element t = (Element) ts.next();
	    String type = t.getAttributeValue("id");
	    ids.add(type);
	    String name = "";
	    if (t.getChildren("name").size()>0)
		name = t.getChildText("name");
	    if (t.getChildren("template").size()>0)
		templates.add(t.getChildText("template"));
	    types.add(name+" ("+type+")");
	    getTypes(t,types,ids,templates);
	}
    }

}

