package uk.ac.ic.doc.y2002.tool.application;

import java.io.*;

/** Bridge class allowing Modules to pass instructions back to the main program, for example to display a sequence of transitions firing.
 * 
 */
public class ModuleBridge {
    DisplayPane display;
    PNAnimator animator;

    /** Initialise the ModuleBridge with a reference to the active DisplayPane.
     *@param display a reference to the active DisplayPane which holds the Document which the module is working on.
     */
    public ModuleBridge(DisplayPane display) {
	this.display = display;
	animator = display.animator;
    }

    /** Signal the program to re-initialise the display from the current state of the DOM.
     * Assumes that the type of net has not changed.
     */
    public void reloadDOM() {
	display.makeLabels();
	System.out.println("DisplayPane::re-initialised.");
    }

    /** Request the program to highlight the object, given its id.
     * All other objects will be un-highlighted.
     * @param id String holding the id of the object to be highlighted.
     */
    public void highlight(String id) {
	animator.highlightPNLabel(id,true);
    }

    /** Request the program to un-highlight all objects.
     */
    public void unHighlight() {
	animator.highlightAll(false);
    }

    /** Request the program to animate a sequence of transitions, given a list of the transition ids.
     * To refer to elements within subnets, may use (e.g.) _s0_p1 for element p1 within subnet s0 
     * (p1 and s0 are ids). The automatic flattening (which the program does when requested) also uses
     * this naming scheme.
     * @param trans_ids An array of transition ids, in order of firing
     * @param length The number of transitions in the sequence
     */
    public void animate(String[] trans_ids, int length) {
	display.eframe.app.setAnimate(display.eframe.inSubnet(),true);
	animator.animate(trans_ids, length);
	System.out.println("ModuleBridge:: module requests animation of "+length+" transitions");
	String transids = "";
	for (int i=0; i<length; i++) {
	    transids+= ", "+trans_ids[i];
	}
	System.out.println("ModuleBridge:: animate sequence: "+transids);
    }

}



