package uk.ac.ic.doc.y2002.tool.application;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import org.jdom.*;
import uk.ac.ic.doc.y2002.tool.editor.*;

public class PNAnimator {
    DisplayPane display;
    java.util.Timer animationTimer;
    Element backup_root=null;
    int timestep = 1000;
    boolean animating=false;
    ArrayList trans_ids = new ArrayList();

    public PNAnimator(DisplayPane display) {
	this.display = display;
	//System.out.println("PNAnimator:: initialised");
    }

    public void setTimestep(int t){
	timestep = t;
	pause();
	highlightAll(false);
	animate();
    }

    public boolean isAnimating(){
	return animating;
    }
    
    /** Makes a backup of the current DOM - but not if there's already a backup stored. */
    protected void makeBackup(){
	// only makes backup if there isn't one stored already.
	if (backup_root == null)
	    backup_root = (Element) display.root.clone();
    }

    protected void restoreFromBackup(){
	// replaces root element in the Document with the backup
	Element docroot = display.eframe.doc.getRootElement();
	docroot.removeContent(display.root);
	docroot.addContent(backup_root);
	// redirects main DisplayPane to the backup DOM.
	// clears the local reference to the DOM
	display.root = backup_root;
	backup_root = null;
    }

    /** Restores main DOM to the backed-up version */
    public void revert(){
	pause();
	int choice;
	choice = JOptionPane.showConfirmDialog(JOptionPane.getDesktopPaneForComponent(display), "Do you want to return the net to the state it was in before animation started?","Ending animation - Revert?", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
	//System.out.println("choice: "+choice);
	if (choice == JOptionPane.CANCEL_OPTION) {
	    //    System.out.println("cancel");
	    // do nothing
	} else {
	    if (choice == JOptionPane.YES_OPTION){
		//System.out.println("yes");
		if (backup_root!=null){
		    //  System.out.println("restoring backup");
		    display.closeSubnetInstances();
		    restoreFromBackup();
		    display.removeAll();
		    display.makeLabels();
		    display.repaint();
		    System.out.println("PNAnimator:: restored initial state of net");
		} else {
		    System.out.println("PNAnimator:: error restoring initial state of net");
		}    
	    } else {
		//System.out.println("no");
		backup_root = null; // clear the backup
		System.out.println("PNAnimator:: keeping new state of net");
	    }
	    animating = false;
	}
	
    }

    public boolean highlightPNLabel(PNLabel label, boolean highlight){
	highlightAll(false);
	highlightPNLabelOnly(label,highlight);
	return true;
    }
    public boolean highlightPNLabel(String id, boolean highlight){
	PNLabel label = display.getLabel(id);
	if (label==null) return false;
	else return highlightPNLabel(label, highlight);
    }

    public boolean highlightPNLabelOnly(PNLabel label, boolean highlight){
        label.repaint();
	label.highlight(highlight);
        label.repaint();
	return true;
    }

    public void highlightAll(boolean highlight){
	HashMap m = display.idTable;
	for (Iterator i=m.entrySet().iterator(); i.hasNext();) {
	    Map.Entry e = (Map.Entry) i.next();
	    PNLabel l = (PNLabel) e.getValue();
	    l.highlight(highlight);
	}
	display.repaint();
    }

    /** Highlights enabled transitions and returns a List of enabled transitions 
     * @return List of enabled transitions (the TransitionLabels) 
     */
    public java.util.List highlightEnabledTransitions(boolean highlight){
	java.util.List trans = new java.util.LinkedList();
	
	// run through the transitions in the current display pane
	HashMap m = display.transitionTable;
	for (Iterator i=m.entrySet().iterator(); i.hasNext();) {
	    Map.Entry e = (Map.Entry) i.next();
	    TransitionLabel l = (TransitionLabel) e.getValue();
	    if (l.isActive()){
		l.highlightEnabled(highlight);
		trans.add(l);
	    } else {
		l.highlightEnabled(!highlight);
	    }
	}
	
	// run through the subnets
	Iterator subs = display.subnetTable.values().iterator();
	while (subs.hasNext()) {
	    SubnetLabel sub = (SubnetLabel) subs.next();
	    DisplayPane subDisplay = sub.instanceFrame.display;
	    java.util.List trans2 = subDisplay.animator.highlightEnabledTransitions(highlight);
	    if (trans2.size()>0) trans.addAll(trans2);
	}
	
	return trans;
    }

    /** Call this function to start the automatic animation of the currently active net. */
    public void animate(){
	animating = true;
	
	// start animationTimer to deal with the automatic animation
	java.util.List trans = new java.util.LinkedList();

	if (animationTimer==null){
	    System.out.println("PNAnimator:: Animating");

	    // save the initial state of the net - make a copy of the DOM
	    makeBackup();

	    animationTimer = new java.util.Timer();
            animationTimer.scheduleAtFixedRate(new TimedHighlight(trans,this,animationTimer),200, timestep*5);    
	} else { 
	    //System.out.println("PNAnimator:: Already animating!");
	}
    }

    /** Call this function to start a particular animation of the currently active net. 
     * @param list_trans String array of transition IDs, in order of firing
     * @param length Number of transitions in the array
     */
    public void animate(String[] list_trans, int length){
	this.trans_ids.clear();
	for (int i=0;i<length;i++){
	    if (list_trans[i]!=null)
		trans_ids.add(i,list_trans[i]);
	}
	animate();
    }

    /** Call this function to stop (pause) any currently running animation. */
    public void pause(){
	if (animationTimer!=null){
	    System.out.println("PNAnimator:: Pausing...");
	    animationTimer.cancel();
	    animationTimer = null;
	} else {
	    //System.out.println("PNAnimator:: Already paused!");
	}
    }

   
    protected TransitionLabel chooseFire(java.util.List trans){
	// chooses a random enabled transition
	if (trans.size()>0){
	    int rand = new Random().nextInt(trans.size());
	    TransitionLabel l = (TransitionLabel) trans.get(rand);
	    System.out.println("PNAnimator:: chosen transition "+l.getID()+" to fire");
	    return l;
	} else {
	    //System.out.println("PNAnimator:: no transition enabled to fire!");
	    return null;
	}
    
    }
    
    // contained classes

    /** A basic TimerTask (subclassed below) */
    class TimedTask extends TimerTask implements Runnable{
	java.util.List trans;
	PNAnimator animator;
	java.util.Timer timer;
	
	public TimedTask(java.util.List trans, PNAnimator animator, java.util.Timer timer){
	    this.trans = trans;
	    this.animator = animator;
	    this.timer = timer;
	}
	public void run(){}
    }

    /** A TimerTask which is called upon by animate() to highlight the Enabled Transitions */
    class TimedHighlight extends TimedTask implements Runnable{
	int i=0;

	public TimedHighlight(java.util.List trans, PNAnimator animator, java.util.Timer timer){
	    super(trans,animator,timer);
	}
	
	public void run(){
	    trans = animator.highlightEnabledTransitions(true);
	                
	    // see if a list of transitions has already been specified
	    if (animator.trans_ids==null || animator.trans_ids.size()<=0){
		// no transitions specified, or have reached end of list
		// choose transition to fire randomly
		//System.out.println("TimedHighlight::no transition list specified");
		animator.trans_ids.clear();
		timer.schedule(new TimedChooseFire(trans,animator,timer),timestep);
	    } else {
		// there is a specified list of transitions, length > 0
		//System.out.println("TimedHighlight:: getting label for transition "+animator.trans_ids.get(0));
		//System.out.println("TimedHighlight:: there are "+animator.trans_ids.size()+" transitions to go");
		TransitionLabel l = (TransitionLabel) animator.display.getLabel((String) animator.trans_ids.get(0));
		if (l==null){
		    // specified transition id is invalid - pause animation.
		    System.out.println("TimedHighlight::specified transition id does not exist in the net.");
		    JOptionPane.showMessageDialog(JOptionPane.getFrameForComponent(display),"Specified transition does not exist in the net! Pausing...", "Invalid animation request",JOptionPane.PLAIN_MESSAGE);   
		    animator.trans_ids.clear();
		    animator.pause();
		} else if (!l.isActive()){
		    // specified transition is not active - pause animation.
		    System.out.println("TimedHighlight::specified transition is not active!");
		    JOptionPane.showMessageDialog(JOptionPane.getFrameForComponent(display),"Specified transition is not active! Pausing...", "Invalid animation request",JOptionPane.PLAIN_MESSAGE);   
		    animator.trans_ids.clear();
		    animator.pause();
		} else {
		    // specified transition is active; ok
		    System.out.println("TimedHighlight:: specified transition to fire "+l.getID());
		    timer.schedule(new TimedChooseFire(trans,animator,timer,l),timestep);
		}
	    }
	}
    }

    /** A TimerTask triggered by TimedHighlight.run() that chooses and highlights the next transition to fire. */
    class TimedChooseFire extends TimedTask implements Runnable{
	TransitionLabel l;

	public TimedChooseFire(java.util.List trans, PNAnimator animator, java.util.Timer timer){
	    super(trans,animator,timer);
	}
	public TimedChooseFire(java.util.List trans, PNAnimator animator, java.util.Timer timer, TransitionLabel l){
	    super(trans,animator,timer);
	    this.l = l;
	}
	
	public void run(){
	    if (l==null){
		//System.out.println("TimedChooseFire:: choosing random transition");
		l = animator.chooseFire(trans);
	    } else {
		//System.out.println("TimedChooseFire:: next transition specified as "+l.getID());
	    }
	    //System.out.println("TimedChooseFire::highlighting chosen transition");
	    if (l==null) {
		// no transitions enabled - deadlock!
		// pause animation.
		System.out.println("TimedChooseFire:: no transitions enabled - Deadlock!");
		JOptionPane.showMessageDialog(JOptionPane.getFrameForComponent(display),"No transitions enabled - Deadlock!", "Deadlock",JOptionPane.PLAIN_MESSAGE);
		animator.pause();
	    } else {
		l.highlightFire();
		timer.schedule(new TimedArcHighlight(l.getInputArcs(),animator,true),timestep);
		timer.schedule(new TimedArcHighlight(l.getInputArcs(),animator,false),timestep*2);
		timer.schedule(new TimedArcHighlight(l.getOutputArcs(),animator,true),timestep*2+1);
		timer.schedule(new TimedArcHighlight(l.getOutputArcs(),animator,false),timestep*3);
		timer.schedule(new TimedFire(l,animator),timestep*3+1);
		// pause before next cycle:
		timer.schedule(new TimedArcHighlight(l.getOutputArcs(),animator,false),timestep*4-20);

	    }
	}
    }
    
    /** A TimerTask triggered by TimedChooseFire.run() that fires the chosen transition. */
    class TimedFire extends TimerTask implements Runnable{
	PNAnimator animator;
	TransitionLabel l;

	public TimedFire(TransitionLabel l, PNAnimator animator){
	    this.l = l;
	    this.animator = animator;
	}
	
	public void run(){
	    //    System.out.println("TimedFire::firing transition "+l.getID());
	    l.fire();
	    if (animator.trans_ids!=null && animator.trans_ids.size()>0) 
		animator.trans_ids.remove(0);    
	}
    }
    /** A TimerTask triggered by TimedChooseFire that highlights certain arcs. */
    class TimedArcHighlight extends TimerTask implements Runnable{
	PNAnimator animator;
	java.util.List arcs = new java.util.LinkedList();
	boolean highlight;

	public TimedArcHighlight(java.util.List arcs, PNAnimator animator, boolean highlight){
	    this.arcs = arcs;
	    this.animator = animator;
	    this.highlight = highlight;
	}
	
	public void run(){
	    //System.out.println("TimedArcHighlight::highlighting labels");
	    Iterator as= arcs.iterator();
	                                
	    while (as.hasNext()) {
		ArcLabel arc = (ArcLabel) as.next();
		arc.highlightLine(highlight);
	    }
	                                
	}
    }
    
}    
    
    
    
    
