package uk.ac.ic.doc.y2002.tool.editor;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;

/** The properties box corresponding to the ArcLabel.
 */
public class ArcPropertiesBox extends PNPropertiesBox {

    public ArcPropertiesBox(Element e) {
        super(e);
    }

    /** Find and add to the panel elements that require special treatment, 
     * and those that should always be available for a particular type of 
     * petri net object.
     * If they're not present, it adds them in with default values.
     * When dealt with, add a reference to the element (if it was the value
     * being displayed) or the attribute to the "specials" HashSet.
     */
    protected void doSpecial(JPanel panel, Element e) {
	super.doSpecial(panel,e);

	String s;
	JTextField field;
	// <... source="id" target="id"> Assume these are present!
	// non-editable (don't want to create inconsistent references)
	Attribute a = e.getAttribute("source");
	field = doAttribute(format("Source id:"),panel,e,a);
	field.setEditable(false);
	field.setEnabled(false);
	field.setToolTipText("The Source element linked to this arc.");
	specials.add(a);
	a = e.getAttribute("target");
	field = doAttribute(format("Target id:"),panel,e,a);
	field.setEditable(false);
	field.setEnabled(false);
	field.setToolTipText("The Target element linked from this arc.");
	specials.add(a);
	
	// arc weight
	// <inscription><value>1</value></inscription>
	if (element.getChildren("inscription",ns).size()<=0)
	    element.addContent(new Element("inscription",ns));
	if (element.getChild("inscription",ns).getChildren("value",ns).size()<=0)
	    element.getChild("inscription",ns).addContent(new Element("value",ns));
	Element inscriptionv = element.getChild("inscription",ns).getChild("value",ns);
	s = inscriptionv.getTextTrim();
	if (s==null || s.equals("")) inscriptionv.setText("1");
	doTextContent(format("Arc weight:"), panel, inscriptionv);
	specials.add(inscriptionv);
	
    }    
    
    /** Find those elements which should not be shown in the properties box,
     * and add them to the "hiddens" HashSet.
     */
    protected void doHidden(JPanel panel, Element e) {
	super.doHidden(panel,e);
	
	Element el;
	
	// <inscription><graphics><offset x="0" y="0" page="1" /></graphics></inscription>
	// ie everything below graphics (except stuff specified in doSpecial)
	if (element.getChildren("inscription",ns).size()>0)
	    if (element.getChild("inscription",ns)
		.getChildren("graphics",ns).size()>0){
		el = element.getChild("inscription",ns).getChild("graphics",ns);
		if (el!=null) hiddens.add(el);   
	    }
	
    } 
    
}





