package uk.ac.ic.doc.y2002.tool.editor;

import javax.swing.JLabel;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*; 
import java.awt.*;
import java.util.*;
import org.jdom.*;
import uk.ac.ic.doc.y2002.tool.application.*;

/** The ArcTypedLabel is a simple extension of ArcLabel which has one 
 * extra attribute - the type of arc.
 * Arc types: normal, read, inhibitor, reset.
 */
public class ArcTypedLabel extends ArcLabel implements MouseListener, MouseMotionListener{
    
    public ArcTypedLabel(Element e) {
	super(e);
	// do custom initialisation
	contextmenu = ArcContextMenu.class;
	propertiesbox = ArcTypedPropertiesBox.class;
    }
 
    public void paintComponent(Graphics g){
	super.paintComponent(g);
	Graphics2D g2 = (Graphics2D)g;
	g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

	g2.setPaint(Color.black);
	
	// Draws the arc weight (if it is not 1 - default)
	String weight = String.valueOf(getWeight());
	if (getWeight()!=1)
	    g2.drawString(weight,centresize/2,centresize+10);
	
	// repaint lines
	line.updateLine();
    }
 
    /** Access function for the arc type stored in the JDOM Document:
     * <arc><type value="inhibitor" /></arc> for example.
     * Defaults to "normal" if unspecified.
     * @return The Arc Type (default "normal")
     */
    public String getType(){
	// if <type> not present, assume default arc type "normal".
	String type = "normal";
	
	if (element.getChildren("type",ns).size()>0) {
	    if (element.getChild("type",ns).getAttributeValue("value")!=null) {
		type = element.getChild("type",ns).getAttributeValue("value");
		if (!type.equals("")) return type;
		else type = "normal";
	    }
	}
	return type;
    }

    /** Sets the Arc Type in the JDOM Document.
     */
    public void setType(String type){
	if (element.getChildren("type",ns).size()<=0) {
	    element.addContent(new Element("type",ns));
	}
	element.getChild("type",ns).setAttribute("value",type);
    }
    
    /** If arc is an "inhibitor" arc, it is only active if its source place
     * has no tokens.
     * Otherwise, this implementation assumes the "normal" arc implementation
     * in ArcLabel.
     * @see ArcLabel#isActive();
     */
    public boolean isActive(){
	if (getType().equals("inhibitor")){
	    try {
		PlaceLabel p = (PlaceLabel) sourcelabel;
		if (p.getMarking()==0) return true;
	    } catch (Exception e){
		System.out.println("ArcLabel::Can't add tokens to a non-place");
	    }    
	    return false;
	} else {
	    return super.isActive();
	}
    }
    
    /** When fired, an Arc adds its weight in Tokens to its target.
     * If the target is not a Place, an error message is printed. 
     * @see #unfire()
     */
    public void fire(){
	try {
	    PlaceLabel p = (PlaceLabel) targetlabel;
	    p.addToken(getWeight());
	} catch (Exception e){
	    System.out.println("ArcLabel::Can't add tokens to a non-place");
	}
    }
    
    /** When un-fired, an Arc removes its weight in Tokens from its source.
     * If the source is not a Place, an error message is printed.
     * @see #fire()
     */
    public void unfire(){
	try {
	    PlaceLabel p = (PlaceLabel) sourcelabel;
	    p.remToken(getWeight());
	} catch (Exception e){
	    System.out.println("ArcLabel::Can't add tokens to a non-place");
	}
    }

    // Function to create a new arc element.
    public static Element makeElement(Integer x, Integer y, String id, String source, String target, Namespace ns, String arctype) {
	Element arc = new Element("arc", ns);
	arc.setAttribute("id", id);
	arc.setAttribute("source", source);
	arc.setAttribute("target", target);
	
	Element graphics = PNLabel.makeGraphics(String.valueOf(x), String.valueOf(y), ns);
	Element inscription = makeInscription(ns);
	
	arc.addContent(inscription);
	arc.addContent(graphics);
	
	Element type = new Element("type",ns);
	type.setAttribute("name",arctype);
	arc.addContent(type);
	
	return arc;
    }
    
    /** Re-use this function to change the arc weight. */
    public void addToken(){
	int weight = getWeight();
	setWeight(++weight);
	repaint();
    }

    /** Re-use this function to change the arc weight. */
    public void remToken(){
	int weight = getWeight();
	if (weight > 1){
	    setWeight(--weight);
	    repaint();
	}
    }
    
}



