package uk.ac.ic.doc.y2002.tool.editor;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;

/** The properties box corresponding to the whole net - managed
 * in the DisplayPane. Obviously, most of the sub-elements must 
 * be hidden!
 */
public class DisplayPropertiesBox extends PropertiesFrame {

    public DisplayPropertiesBox(Element e) {
        super(e);
    }

    /** Find and add to the panel elements that require special treatment, 
     * and those that should always be available for a particular type of 
     * petri net object.
     * If they're not present, it adds them in with default values.
     * When dealt with, add a reference to the element (if it was the value
     * being displayed) or the attribute to the "specials" HashSet.
     */
    protected void doSpecial(JPanel panel, Element e) {
	super.doSpecial(panel,e);

	String s;
	JTextField field;
	Attribute a;

	// name of net/module
	// <name><value>textname</value></name>
	if (element.getChildren("name",ns).size()<=0)
	    element.addContent(new Element("name",ns));
	if (element.getChild("name",ns).getChildren("value",ns).size()<=0)
	    element.getChild("name",ns).addContent(new Element("value",ns));
	Element namev = element.getChild("name",ns).getChild("value",ns);
	s = namev.getTextTrim();
	if (s==null) namev.setText("");
	field = doTextContent(format("Name:"), panel, namev);
	field.setToolTipText("Descriptive Name of the element: not the same as the unique id!");
	specials.add(namev);

	// if <module>, check for <module name="mname">
	// assume it exists (modules must be able to be referenced!)
	if (element.getName().equals("module")){
	    a = element.getAttribute("name");
	    s = element.getAttributeValue("name");
	    field = doAttribute(format("Module id reference:"),panel,element,a);
	    field.setToolTipText("Module id used to reference this module");
	    field.setEnabled(false);
	    specials.add(a);
	}

	// net type  <net id="n1" type="ptNet">
	if (e.getAttribute("type")==null) { // attribute not present
	    e.setAttribute("type","ptNet");
	}
	a = e.getAttribute("type");
	s = e.getAttributeValue("type");
	if (s.equals("")) a.setValue("ptNet");
	field = doAttribute(format("Net Type:"),panel,e,a);
	
	field.setToolTipText("Type of Net");
	field.setEnabled(false);
	specials.add(a);

	// net id  <net id="n1" type="ptNet">
	if (element.getName().equals("net")){
	    if (e.getAttribute("id")==null) { // attribute not present
		e.setAttribute("id","ptNet");
	    }
	    a = e.getAttribute("id");
	    s = e.getAttributeValue("id"); // there should be an id!
	    field = doAttribute(format("Unique ID:"),panel,e,a);
	    
	    field.setToolTipText("Unique ID of net");
	    field.setEnabled(false);
	    specials.add(a);
	}
	
    }    
    
    /** Find those elements which should not be shown in the properties box,
     * and add them to the "hiddens" HashSet.
     */
    protected void doHidden(JPanel panel, Element e) {
	super.doHidden(panel,e);
	
	Element el;
	Iterator els= e.getChildren().iterator();
	String elname = "";
	while (els.hasNext()) {
	    el = (Element) els.next();
	    elname = el.getName();
	    if (elname.equals("place")
		|| elname.equals("transition")
		|| elname.equals("arc")
		|| elname.equals("page")
		|| elname.equals("referencePlace")
		|| elname.equals("referenceTransition")
		|| elname.equals("importPlace")
		|| elname.equals("exportPlace")
		|| elname.equals("importTransition")
		|| elname.equals("exportTransition")
		|| elname.equals("instance")
		|| elname.equals("interface")){
		hiddens.add(el);
	    }
	}

    } 
    
}





