package uk.ac.ic.doc.y2002.tool.editor;

import javax.swing.JLabel;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.*;
import java.util.*;
import org.jdom.*;

/** The LineLabel draws the actual lines and arrow in the arc. 
 *  It is in a separate class from the ArcLabel because we don't 
 *  want the whole rectangular label that contains the line to 
 *  be click-able - only the region around the arrow. 
 */
public class LineLabel extends JLabel {
    int sx,sy,tx,ty,ax,ay;
    int xmin,xmax,ymin,ymax; // bounds
    PNLabel source, target, arc;
    double rotation=0;
    protected Color fillcolour = Color.black;
    protected Color normalfill = Color.black;
    protected Color highlightfill = Color.red;

    public LineLabel(PNLabel s, PNLabel a, PNLabel t) {
	super();
	source = s;
	target = t;
	arc = a;
	updateLine();
	calcBounds();
    }

    public void paintComponent(Graphics g){
	super.paintComponent(g);
	Graphics2D g2 = (Graphics2D)g;

	g2.setPaint(fillcolour);

	g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
	g2.setStroke(new BasicStroke(1.5f));
	
	// draw the lines
	Line2D line = new Line2D.Double(sx-getX(), sy-getY(), ax-getX(), ay-getY());
	g2.draw(line);
	line = new Line2D.Double(ax-getX(), ay-getY(), tx-getX(), ty-getY());
	g2.draw(line);

	// draw the arrow, rotated
	calcRotation();

	// save original transform settings
	AffineTransform saveXform = g2.getTransform();
	// set up this Label's transform
	AffineTransform at = new AffineTransform();
	at.rotate(rotation,ax-getX(),ay-getY()); 
	g2.transform(at);

	g2.setStroke(new BasicStroke(1.0f));

	line = new Line2D.Double(ax-getX()-5, ay-getY()+10, ax-getX(), ay-getY());
	g2.draw(line);
	line = new Line2D.Double(ax-getX()+5, ay-getY()+10, ax-getX(), ay-getY());
	g2.draw(line);

	// restore original transform settings
	g2.setTransform(saveXform);
    }

    public void updateLine() {
	sx = source.getJX();
	sy = source.getJY();
	tx = target.getJX();
	ty = target.getJY();
	ax = arc.getJX();
	ay = arc.getJY();
	calcBounds();
    }
    
    protected void calcBounds() {
	xmin = Math.min(sx,tx);  xmin = Math.min(xmin,ax);
	xmax = Math.max(sx,tx);xmax = Math.max(xmax,ax);
	ymin = Math.min(sy,ty);ymin = Math.min(ymin,ay);
	ymax = Math.max(sy,ty);ymax = Math.max(ymax,ay);
	setBounds(xmin-10,ymin-10,xmax+10,ymax+10);
    }

    protected void calcRotation() {
	double angle = Math.atan( ( (double)tx-(double)sx)/( (double)sy-(double)ty));
	//System.out.println("LineLabel::angle is "+angle+ " - "+arc.getID());
	
	// first check for cases where line is horizontal
	if ((angle>-1.58 && angle < -1.56) || (angle>1.56 && angle<1.58)){
	    //System.out.println("LineLabel:: +-1.57");
	    if (sx>tx){
		//System.out.println("LineLabel:: Source is right of target");
		angle = Math.toRadians(-90);
	    } else {
		//System.out.println("LineLabel:: Source is left of target");
		angle = Math.toRadians(90);
	    }
	// otherwise, determine rotation by quadrant
	} else {
	    if (tx>=sx && ty>=sy) {
		// rotate by 180 degrees
		angle += Math.toRadians(180);
	    } else if (sx>=tx && sy>=ty) {
		// ok as is
	    } else if (tx<sx && ty>=sy) {
		// rotate by 180 degrees
		angle += Math.toRadians(180);
	    } else if (sx<tx && sy>=ty) {
		// ok as is
	    }
	}
	rotation = angle;
    }

    public void highlight(boolean highlight){
	if (highlight)
	    fillcolour = highlightfill;
	else fillcolour = normalfill;
	repaint();
    }


}


