package uk.ac.ic.doc.y2002.tool.editor;

import javax.swing.JLabel;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*; 
import java.awt.*;
import java.util.*;
import org.jdom.*;
import uk.ac.ic.doc.y2002.tool.application.*;

/** Superclass for node elements in the Petri net, such as places and transitions. 
 * This class should not be instantiated.
 */
public class NodeLabel extends PNLabel implements MouseListener, MouseMotionListener{
    /** List of ArcLabels connected as inputs to this NodeLabel */
    java.util.List inputArcs = new java.util.LinkedList();
    /** List of ArcLabels connected as outputs from this NodeLabel */
    java.util.List outputArcs = new java.util.LinkedList();
    /** Pointer to NodeLabel target of which this label is source */
    NodeLabel refNodeTarget = null;
    /** List of NodeLabels for which this label is a target */
    java.util.List refNodeSources = new java.util.LinkedList();

    public NodeLabel(Element e) {
	super(e);
	// set up member variables
	offset = 4;
	// do custom initialisation
	updatePosition();
	propertiesbox = NodePropertiesBox.class;
    }

    /** Called when a label is asked to remove itself from the DisplayPane
     *  @return DisplayPane
     */
    public DisplayPane remove(){
	
	System.out.println("removing label: " + getID());

	DisplayPane display = getDisplayPane();

	// remove element from JDOM.
	Element my_parent = element.getParent();
	my_parent.removeContent(element);
	
	// If PNLabel is a source for a reference link.
	String referenceID = element.getAttributeValue("ref");
	String elementName = element.getName();
	if (referenceID != null) {
	    removeReferenceLabel();
	    removeTargetSourceEntry(this);
	}
     
	// if target for a reference link.
	if (referenceTarget()) {
	    notifySource();
	    java.util.List sourceList = new LinkedList(getRefNodeSources());
	    Iterator it = sourceList.iterator();
	        
	    while (it.hasNext()) {
		NodeLabel source = (NodeLabel) it.next();
		source.removeRefNodeTarget();
		removeRefNodeSource(source);
	    }
	}

	// remove PNLabel from DisplayPane it is in.
	display.remove(this);
	
	return display;
    }
 
    public void removeTargetSourceEntry(NodeLabel source) {

	// remove refNode Target and Source.
	NodeLabel node_target = getRefNodeTarget();
	node_target.removeRefNodeSource(source);
	removeRefNodeTarget();
    }


    // function to remove a labels reference link and hashmap entries.
    public void removeReference() {

	element.removeAttribute("ref");
    }

    // return true if pointed to by a reference link.
    public boolean referenceTarget() {

	java.util.List labels = getRefNodeSources();

	if (labels != null)
	    return true;

	return false;
    }

    // Tell source to remove reflink
    public void notifySource() {
	
	java.util.List sourceList = getRefNodeSources();
	Iterator it = sourceList.iterator();
	while (it.hasNext()) {

	    NodeLabel source = (NodeLabel) it.next();
	    if (source != null)
		source.removeReferenceLabel();
	}
    }

    
    // find reflinklabel and ask it to remove itself.
    public void removeReferenceLabel() { 
	
	DisplayPane display = getDisplayPane();
	HashMap refLinkTableCopy = new HashMap(display.refLinkTable);

	RefLinkLabel label = (RefLinkLabel) refLinkTableCopy.get(this);
	
	// ask label to remove itself.
	if (label != null) {
	    label.remove();
	    removeReference();
	}
	
	display.refLinkTable.remove(this);
    }

    /** Function to create a name element used by NodeLabel elements (places and transitions etc.).
     *  @return Element
     */
    public static Element makeName(String myName, Namespace ns) {

	// name element
	Element name = new Element("name", ns);
	Element n_graphic = new Element("graphics", ns);
	Element n_g_offset = new Element("offset", ns);
	Element n_value = new Element("value", ns);
	
	n_g_offset.setAttribute("page", "1");
	n_g_offset.setAttribute("x", "-20");
	n_g_offset.setAttribute("y", "40");
	n_value.setText(myName);
	
	n_graphic.addContent(n_g_offset);
	name.addContent(n_graphic);
	name.addContent(n_value);

	return name;
    }

    public void paintComponent(Graphics g){
        super.paintComponent(g);
    }
    
    /** @return name of Petri Net object (not necessarily the same as the id) */
    public String getName() {
	String name="";
	if ( element.getChildren("name",ns).size()>0 && element.getChild("name",ns).getChildren("value",ns).size() > 0) {
	    name = element.getChild("name",ns).getChildTextTrim("value",ns);
	}
	return name;
    }

    /** typically called when a new refLink is added to the net. */
    public void addRefNodeTarget(NodeLabel node) {
	//System.out.println("NodeLabel:: added ref target "+node.getID()+" to "+getID());
	refNodeTarget = node;
    }
    /** typically called when a new refLink is added to the net. */
    public void addRefNodeSource(NodeLabel node) {
	//System.out.println("NodeLabel:: added ref source "+node.getID()+" to "+getID());
	refNodeSources.add(node);
    }
    /** returns a list of refNodeSource elements
     *  @return java.util.List
     */
    public java.util.List getRefNodeSources() {
	return refNodeSources;
    }
    /** returns the refNodeTarget element
     *  @return NodeLabel
     */
    public NodeLabel getRefNodeTarget() {
	return refNodeTarget;
    }
    /** typically called when a refLink is removed from the net. */
    public void removeRefNodeTarget() {
	refNodeTarget = null;
    }
    /** typically called when a refLink is removed from the net. */
    public void removeRefNodeSource(NodeLabel node) {
	refNodeSources.remove(node);
    }
    /** typically called when a new arc is added to the net. */
    public void addInputArc(ArcLabel a){
	inputArcs.add(a);
	//System.out.println("added input arc");
    }
    /** typically called when a new arc is added to the net. */
    public void addOutputArc(ArcLabel a){
	outputArcs.add(a);
	//System.out.println("added output arc");
    }
    /** typically called when an arc is removed from the net. 
     * @see ArcLabel 
     */
    public void removeOutputArc(ArcLabel a){
	outputArcs.remove(a);
    }
    /** typically called when an arc is removed from the net. 
     * @see ArcLabel 
     */
    public void removeInputArc(ArcLabel a){
	inputArcs.remove(a);
    }
    /** empties the local list of references to input ArcLabels */
    public void clearInputArcs(){
	inputArcs.clear();
    }
    /** empties the local list of references to output ArcLabels */
    public void clearOutputArcs(){
	outputArcs.clear();
    }
    /** returns the local list of references to input ArcLabels */
    public java.util.List getInputArcs(){
	return inputArcs;
    }
    /** returns the local list of references to output ArcLabels */
    public java.util.List getOutputArcs(){
	return outputArcs;
    }

    /** A new version of isActive() - allows more specific
     * queries, taking arc weights and numbers of tokens into account.
     * @see PlaceLabel
     */
    public boolean isActive(int number){
	return isActive();
    }
    
    /** A node is active if all of its input arcs are active -
     * this function queries the arcs.
     */
    public boolean isActive(){
	// Node is active if all its input arcs are active
	Iterator inputs= inputArcs.iterator();

	while (inputs.hasNext()) {
	    ArcLabel arc = (ArcLabel) inputs.next();
	    if (!arc.isActive()) return false;
	}

	return true;
    }

    /** Updates DOM with new values and sets the label position -  
     *  - also updates connected arcs
     */
    public void setPosition(int x, int y) {
	super.setPosition(x,y);

	Iterator arcs= inputArcs.iterator();
	while (arcs.hasNext()) {
	    ArcLabel arc = (ArcLabel) arcs.next();
	    arc.positionLines();
	}
	arcs = outputArcs.iterator();
	while (arcs.hasNext()) {
	    ArcLabel arc = (ArcLabel) arcs.next();
	    arc.positionLines();
	}
    }

    // returns a list of PNLabels which are linked to this PNLabel
    public java.util.List getLinkedLabels(){
	java.util.List linked = new java.util.LinkedList();
	linked.addAll(inputArcs);
	linked.addAll(outputArcs);
	return linked;
    }

    public void updateReferences(){
	Iterator refs= refNodeSources.iterator();

	while (refs.hasNext()) {
	    NodeLabel ref = (NodeLabel) refs.next();
	    ref.repaint();
	    ref.updateReferences();
	}
    }
}












