package uk.ac.ic.doc.y2002.tool.editor;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;

/** The properties box corresponding to the PNLabel.
 * This contains common functionality, but in most
 * cases a more specific subclass will exist for each
 * type of label.
 */
public class PNPropertiesBox extends PropertiesFrame {

    public PNPropertiesBox(Element e) {
        super(e);
    }

    /** Find and add to the panel elements that require special treatment, 
     * and those that should always be available for a particular type of 
     * petri net object.
     * If they're not present, it adds them in with default values.
     * When dealt with, add a reference to the element (if it was the value
     * being displayed) or the attribute to the "specials" HashSet.
     */
    protected void doSpecial(JPanel panel, Element e) {
	String s;
	JTextField field;
	
	// <... id="id"> Assume this is present! in trouble if it's not.
	// non-editable (don't want to create inconsistent references)
	Attribute id = e.getAttribute("id");
	field = doAttribute(format("Unique id:"),panel,e,id);
	field.setEditable(false);
	field.setEnabled(false);
	field.setToolTipText("The unique id of this element - not editable.");
	specials.add(id);
	
	// <graphics><position x="0" y="0" /></graphics>
	// sets fields to non-editable, because otherwise it messes up 
	// the display.
	Element position;
	Attribute x, y;
	if (element.getChildren("graphics",ns).size()<=0)
	    element.addContent(new Element("graphics",ns));
	if (element.getChild("graphics",ns).getChildren("position",ns).size()<=0)
	    element.addContent(new Element("position",ns));
	position = element.getChild("graphics",ns).getChild("position",ns);
	s = position.getAttributeValue("x");
	if (s==null || s.equals("")) position.setAttribute("x","0");
	s = position.getAttributeValue("y");
	if (s==null || s.equals("")) position.setAttribute("y","0");
	
	x = position.getAttribute("x");
	y = position.getAttribute("y");

	field = doAttribute(format("X position:"),panel,position,x);
	field.setEditable(false);
	field.setEnabled(false);
	field.setToolTipText("To change the position, move the element onscreen.");
	field = doAttribute(format("Y position:"),panel,position,y);
	field.setEditable(false);
	field.setEnabled(false);
	field.setToolTipText("To change the position, move the element onscreen.");
	
	specials.add(x);
	specials.add(y);
	
    }    
    
    /** Find those elements which should not be shown in the properties box,
     * and add them to the "hiddens" HashSet.
     */
    protected void doHidden(JPanel panel, Element e) {
	Attribute a;
	
	// <graphics><position page="1" /></graphics>
	if (element.getChildren("graphics",ns).size()>0)
	    if (element.getChild("graphics",ns)
		.getChildren("position",ns).size()>0){
		a = element.getChild("graphics",ns)
		    .getChild("position",ns)
		    .getAttribute("page");
		if (a!=null) hiddens.add(a);
	    }

	// <toolspecific tool="x" version="x"></toolspecific>
	if (element.getChildren("toolspecific",ns).size()>0){
	    a = element.getChild("toolspecific",ns).getAttribute("tool");
	    if (a!=null) hiddens.add(a);
	    a = element.getChild("toolspecific",ns).getAttribute("version");
	    if (a!=null) hiddens.add(a);
	}
    } 
    

}





