package uk.ac.ic.doc.y2002.tool.editor;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*; 
import java.awt.*;
import java.util.*;
import org.jdom.*;
import uk.ac.ic.doc.y2002.tool.application.*;

/** The PetriContextMenu is the menu that appears when the user
 * right-clicks on a Petri net element.
 * The default implementation provides a Properties option, 
 * which brings up the default Properties box.
 */
public class PetriContextMenu extends JPopupMenu{
    JMenuItem propertiesItem;
    PNLabel label;
    DisplayPane display;
    Class properties_class;

    /** @param l The PetriContextMenu keeps a reference to the PNLabel which it describes.
     * @param prop_class The PetriContextMenu needs to know what type of Properties box to create - this is determined by the calling PNLabel when instantiating the PetriContextMenu.
     */
    public PetriContextMenu(PNLabel l, Class prop_class) {
	this.label = l;
	properties_class = prop_class;
	buildMenu();
    }

    /** @param l Alternative constructor: The PetriContextMenu keeps a reference to the DisplayPane which it describes.
     * @param prop_class The PetriContextMenu needs to know what type of Properties box to create - this is determined by the calling DisplayPane when instantiating the PetriContextMenu.
     */
    public PetriContextMenu(DisplayPane d, Class prop_class) {
	display = d;
	properties_class = prop_class;
	buildMenu();
    }
    
    /** This function should be overridden (with the parent version 
     * called first) in subclasses, to add more menu items.
     */ 
    protected void buildMenu(){
	addProperties();
    }
    
    protected Element getElement(){
	if (label!=null){
	    return label.element;
	} else if (display!=null){
	    return display.root;
	}
	return null;
    }

    /** The Listener which creates the Properties box must use
     * reflection, as the class of the Properties box is not
     * known until runtime - it is determined by the type of net.
     */
    protected void addProperties(){
	propertiesItem = new JMenuItem("Properties");
	propertiesItem.addActionListener( new ActionListener() {
		public final void actionPerformed(ActionEvent e) {
		    // make the Properties popup
		    try {
			Object o = properties_class.getConstructor(new Class[] {Element.class})
			    .newInstance(new Object[] {getElement()});
			PropertiesFrame props = (PropertiesFrame) o;
			//System.out.println("PetriContextMenu::Created new PropertiesFrame of class "+properties_class.getName());

			props.setVisible(true);
			// select the parent JDesktopFrame and add the Properties popup to it
			Container mainframe;
			if (label!=null)
			    mainframe = label.getParent();
			else
			    mainframe = display.getParent();
			Class c = mainframe.getClass();
			while (c != JDesktopPane.class) {
			    mainframe = mainframe.getParent();
			    c = mainframe.getClass();
			    //System.out.println("PetriContextMenu::properties frame is at:"+c.getName());
			}
			mainframe.add(props);
			try {
			    props.setSelected(true);
			} catch (java.beans.PropertyVetoException ex) {}
			
			// feedback to label to allow it to refresh itself, eg
			// modify screen display in response to changes
			// (JDOM gets updated automatically by the Properties box)
			props.addInternalFrameListener(
			   new InternalFrameListener() {
				   public void internalFrameActivated(InternalFrameEvent e){};
				   public void internalFrameClosing(InternalFrameEvent e){};
				   public void internalFrameClosed(InternalFrameEvent e){
				       if (label!=null){
					   label.updateSelf();
				       }
};
				   public void internalFrameDeactivated(InternalFrameEvent e){
				       if (label!=null){
					   label.updateSelf();
				       }
				   }
				   public void internalFrameDeiconified(InternalFrameEvent e){};
				   public void internalFrameIconified(InternalFrameEvent e){};
				   public void internalFrameOpened(InternalFrameEvent e){};
			       }
			   );
		    } catch (Exception ex) {
			System.out.println("PetriContextMenu::Error creating Properties pop-up box! "+e);
		    }
		}
	    });
	add(propertiesItem);

    }
}

