package uk.ac.ic.doc.y2002.tool.editor;

import javax.swing.JLabel;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*; 
import java.awt.*;
import java.util.*;
import org.jdom.*;

/** The PlaceLabel represents a place on screen.
 * It has access functions for the number of tokens on the place (the Marking).
 * It can also be tested to see if it is active, given a number of required tokens.
 */
public class PlaceLabel extends NodeLabel implements MouseListener, MouseMotionListener{
    protected int radius = 20; // PlaceLabel doesn't use the height, width variables
    protected int tokenradius = 3;

    public PlaceLabel(Element e) {
	super(e);

	// set up member variables
	xoffset = radius + offset;
	yoffset = radius + offset;
	preferredSize = new Dimension(2*(radius+offset),2*(radius+offset));
	// do custom initialisation
	contextmenu = PlaceContextMenu.class;
	propertiesbox = PlacePropertiesBox.class;	
    }

    public void updateBounds() {
	setBounds(lastX-(radius+offset),lastY-(radius+offset),2*(radius+offset)+textspace/3,2*(radius+offset)+textspace);

    }

    /** Gets the Marking value from the JDOM Document (the number
     * of tokens on the place). Defaults to 0.
     */
    public int getMarking() {
	if (element.getChildren("initialMarking",ns).size()>0) {
	    if (element.getChild("initialMarking",ns).getChildren("value",ns).size() > 0) {
		String ts = element.getChild("initialMarking",ns).getChild("value",ns).getText();
		int m = Integer.parseInt(ts);
		return m;
	    }
	}
	return 0; // if unspecified, return 0 tokens
    }
 
    /** Sets the Marking value in the JDOM Document.
     * Protected: usually addToken and remToken should be used.
     * @see #addToken()
     * @see #remToken()
     */
    protected void setMarking(int t) {
	if (element.getChildren("initialMarking",ns).size()>0) {
	    if (element.getChild("initialMarking",ns).getChildren("value",ns).size()>0) {
		element.getChild("initialMarking",ns).getChild("value",ns).setText(String.valueOf(t));
	    }
	} else {
	    // TODO: create InitialMarking, Value tags if not there already.
	}
    }

    /** Paints the Circle and the Tokens. 
     * @see #drawTokens()
     */
    public void paintComponent(Graphics g){
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D)g;
	g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
	// Draws the circle.
	g2.setStroke(new BasicStroke(2.0f));
	g2.setPaint(fillcolour);
	g2.fill (new Ellipse2D.Double(offset, offset, 2*radius, 2*radius));
	g2.setPaint(Color.black);
	g2.draw (new Ellipse2D.Double(offset, offset, 2*radius, 2*radius));
	// Draws the tokens.
	drawTokens(g2);
	// Draws the name.
	paintName(g2);
    }

    protected void paintName(Graphics2D g2){
	// Draws the name.
	String name = getName();
	g2.drawString(name,radius/3,2*(radius+offset*2));
	String id = getID();
	if (!name.equals(id)){
	    name = "("+id+")";
	    g2.drawString(name,radius/2,2*(radius+offset*2)+13);
	}
    }
    
    /** Draws tokens: hardcoded for 0 - 9; any higher simply prints the number. */
    protected void drawTokens(Graphics2D g2){
	int tokenno = getMarking();
	int centre = radius +offset - tokenradius;

	g2.setPaint(Color.black);

	if (tokenno==0){
	    // don't draw anything
	} else if (tokenno==1){
	    drawToken(g2,centre,0,0);
	} else if (tokenno==2) {
	    drawToken(g2,centre,tokenradius*2,0);
	    drawToken(g2,centre,-tokenradius*2,0);
	} else if (tokenno==3) {
	    drawToken(g2,centre,tokenradius*2,-tokenradius*2);
	    drawToken(g2,centre,-tokenradius*2,-tokenradius*2);
	    drawToken(g2,centre,0,+tokenradius*2);
	} else if (tokenno==4) {
	    drawToken(g2,centre,-tokenradius*2,-tokenradius*2);
	    drawToken(g2,centre,+tokenradius*2,-tokenradius*2);
	    drawToken(g2,centre,-tokenradius*2,+tokenradius*2);
	    drawToken(g2,centre,+tokenradius*2,+tokenradius*2);
	} else if (tokenno==5) {
	    drawToken(g2,centre,0,-tokenradius*4);
	    drawToken(g2,centre,-tokenradius*3,-tokenradius*1);
	    drawToken(g2,centre,+tokenradius*3,-tokenradius*1);
	    drawToken(g2,centre,-tokenradius*2,+tokenradius*3);
	    drawToken(g2,centre,+tokenradius*2,+tokenradius*3);
	} else if (tokenno==6) {
	    drawToken(g2,centre,-tokenradius*3,-tokenradius*2);
	    drawToken(g2,centre,0,-tokenradius*2);
	    drawToken(g2,centre,+tokenradius*3,-tokenradius*2);
	    drawToken(g2,centre,-tokenradius*3,+tokenradius*2);
	    drawToken(g2,centre,0,+tokenradius*2);
	    drawToken(g2,centre,+tokenradius*3,+tokenradius*2);
	} else if (tokenno==7) {
	    drawToken(g2,centre,tokenradius/2,-tokenradius*3);
	    drawToken(g2,centre,-tokenradius*2,-tokenradius*2);
	    drawToken(g2,centre,+tokenradius*3,-tokenradius*2);
	    drawToken(g2,centre,-tokenradius*3,+tokenradius);
	    drawToken(g2,centre,+tokenradius*4,+tokenradius);
	    drawToken(g2,centre,-tokenradius,+tokenradius*3);
	    drawToken(g2,centre,+tokenradius*2,+tokenradius*3);
	} else if (tokenno==8) {
	    drawToken(g2,centre,-tokenradius*3,-tokenradius*3);
	    drawToken(g2,centre,0,-tokenradius*3);
	    drawToken(g2,centre,+tokenradius*3,-tokenradius*3);
	    drawToken(g2,centre,-tokenradius*3,0);
	    drawToken(g2,centre,+tokenradius*3,0);
	    drawToken(g2,centre,-tokenradius*3,+tokenradius*3);
	    drawToken(g2,centre,0,+tokenradius*3);
	    drawToken(g2,centre,+tokenradius*3,+tokenradius*3);
	} else if (tokenno==9) {
	    drawToken(g2,centre,-tokenradius*3,-tokenradius*3);
	    drawToken(g2,centre,0,-tokenradius*3);
	    drawToken(g2,centre,+tokenradius*3,-tokenradius*3);
	    drawToken(g2,centre,-tokenradius*3,0);
	    drawToken(g2,centre,0,0);
	    drawToken(g2,centre,+tokenradius*3,0);
	    drawToken(g2,centre,-tokenradius*3,+tokenradius*3);
	    drawToken(g2,centre,0,+tokenradius*3);
	    drawToken(g2,centre,+tokenradius*3,+tokenradius*3);
	} else {
	    String tokenstring = String.valueOf(tokenno);
	    g2.drawString(tokenstring,radius-5,radius+offset*2);
	}
    }

    /** Used in drawTokens() 
     * @see #drawTokens(Graphics2D g2)
     */
    protected void drawToken(Graphics2D g2, int centre, int xoffset, int yoffset){
	g2.fill (new Ellipse2D.Double(centre+xoffset,centre+yoffset, 2*tokenradius, 2*tokenradius));
    }
    /** Function to create a new place element.
     * @author Daniel Justice
     */
    public static Element makeElement(Integer x, Integer y, String id, Namespace ns) {

	Element place = new Element("place", ns);
	place.setAttribute("id", id);

	Element graphics = PNLabel.makeGraphics(String.valueOf(x), String.valueOf(y), ns);
	Element name = NodeLabel.makeName(id, ns);
	Element inMark = makeMarking(ns);
	
	place.addContent(name);
	place.addContent(inMark);
	place.addContent(graphics);
	
	return place;
    }
    
    /** Function to create an initial marking used by new places.
     * @author Daniel Justice
     */
    public static Element makeMarking(Namespace ns) {

     
	// initial marking element
	Element inMark = new Element("initialMarking", ns);
	Element i_graphic = new Element("graphics", ns);
	Element i_g_offset = new Element("offset", ns);
	Element i_value = new Element("value", ns);

	i_g_offset.setAttribute("page", "1");
	i_g_offset.setAttribute("x", "-20");
	i_g_offset.setAttribute("y", "10");
	i_value.setText("0");

	i_graphic.addContent(i_g_offset);
	inMark.addContent(i_graphic);
	inMark.addContent(i_value);

	return inMark;
    }
    
    /** Queries if a place is active.
     * @param tokenno The number of tokens that the place must have to be active.
     */
    public boolean isActive(int tokenno){
	if (getMarking() >= tokenno)
	    return true;
	else 
	    return false;
    }
    /** Should not be called on this class. Use isActive(int tokenno) instead.
     * @return false
     */
    public boolean isActive(){
	System.out.println("PlaceLabel:: invalid use of isActive()");
	return false;
    }

    /** Use this function to add a token to the place.
     * @see #addToken(int number)
     * @see #remToken()
     * @see #remToken(int number)
     */
    public void addToken(){
	int tokens = getMarking();
	// add token
	setMarking(++tokens);
	repaint();
	updateReferences();
    }
    /** Use this function to add a specified number of tokens to the place.
     * @see #addToken()
     * @see #remToken()
     * @see #remToken(int number)
     */
    public void addToken(int number){
	int tokens = getMarking();
	// add tokens
	setMarking(tokens+number);
	repaint();
	updateReferences();
    }
    
    /** Use this function to remove a token from the place.
     * @see #remToken()
     * @see #addToken()
     * @see #addToken(int number)
     */
    public void remToken(){
	int tokens = getMarking();
	//remove token
	if (tokens > 0){
	    setMarking(--tokens);
	    repaint();
	    updateReferences();
	}
    }

    /** Use this function to remove the specified number of tokens from the place.
     * @see #remToken(int number)
     * @see #addToken()
     * @see #addToken(int number)
     */
    public void remToken(int number){
	int tokens = getMarking();
	//remove token
	if (tokens >= number){
	    setMarking(tokens - number);
	    repaint();
	    updateReferences();
	}

    }

}










