package uk.ac.ic.doc.y2002.tool.editor;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.beans.*;
import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;

/** The base class for the Properties Window. 
 * The default implementation extracts all elements and
 * attributes below the element being examined.
 * The one extra feature is that it makes "id" fields un-editable.
 */
public class PropertiesFrame extends JInternalFrame {
    Element element;
    Namespace ns;
    static final int xOffset = 400, yOffset = 100;
    HashSet specials = new HashSet();
    HashSet hiddens = new HashSet();
    
    public PropertiesFrame(Element e) {
        super("Properties",
              true, //resizable
              true, //closable
              false, //maximizable
              true);//iconifiable);
	element = e;
	ns = element.getNamespace();
	
        //...Create the GUI and put it in the window...
	JPanel panel = makePanel();
        Container contentPane = getContentPane();
	contentPane.add(panel);
	
        //...Then set the window size or call pack...
        pack();
	
	if (getWidth()<250){
	    setSize(250,getHeight());
	}
	if (getWidth()>400){
	    setSize(400,getHeight());
	}
	if (getHeight()>500){
	    setSize(getWidth(),500);
	}

        //Set the window's location.
        setLocation(xOffset, yOffset);
	
	String title = element.getName();
	
	if (element.getChild("name",ns)!=null)
	    if (element.getChild("name",ns).getChild("value",ns)!=null)
		if (element.getChild("name",ns).getChild("value",ns).getText()!="")
		    title += " "+element.getChild("name",ns).getChild("value",ns).getText();
	if (title.length()>0)
	    setTitle("Properties for: "+title);

	// stops the properties box showing as maximised - problem is,
	// it un-maximises all other frames as well. but this is better
	// than a giant properties box filling the screen.    
	addVetoableChangeListener(new VetoableChangeListener() {
		public void vetoableChange(PropertyChangeEvent evt)
		    throws PropertyVetoException {
		    //System.out.println("Property change!");
		    if (evt.getPropertyName().equals(JInternalFrame.IS_MAXIMUM_PROPERTY)){
			//System.out.println("maximised!");
			throw new PropertyVetoException("don't maximise the properties box",evt);
		    }
		}
	    });
	
    }

    /** Makes the panel which contains the input fields.
     * doElement() creates the input fields by examining the Element.
     * @see doElement(JPanel panel, Element e, String ancestry)
     */
    protected JPanel makePanel() {
	JPanel panel = new JPanel();
	panel.setLayout(new GridLayout(0,2));
	panel.setBorder(BorderFactory.createEmptyBorder(
							10, //top
							10, //left
							10, //bottom
							10) //right
			);
	
	doSpecial(panel, element);
	doHidden(panel, element);
	doElement(panel, element, "");
	
	return panel;
    }

    /** Function which recursively works through the Element
     * and its children, producing labels and input fields. 
     * @see doTextContent(JPanel panel, Element e, String ancestry)
     * @see doAttributes(JPanel panel, Element e, String ancestry)
     */
    protected void doElement(JPanel panel, Element e, String ancestry) {

	// Only do this element if it's not done already
	if (!done(e)) {
	    // build up the label for the entry
	    if (ancestry.length()>0) {
		ancestry += "."+e.getName();
	    } else {
		ancestry = " ";
	    }
	    if (ancestry.startsWith(" .")) {
		ancestry = ancestry.substring(2);
	    }
	                    
	    // deal with simple contained values
	    doTextContent(panel,e,ancestry);
	    // 
	    doAttributes(panel,e,ancestry);
	                    
	    java.util.List contents = e.getChildren();
	    if (contents.size()>0) {
		Iterator cs= contents.iterator();
		while (cs.hasNext()) {
		    Element c = (Element) cs.next();
		    doElement(panel, c, ancestry);
		}
	    }
	}
	
    }
   
    /** Deals with text content within an element, eg
     *<marking>34</marking> - creates a label and a field,
     *  with attached Listener of class ValueUpdater.
     *  @see #ValueUpdater
     */
    protected void doTextContent(JPanel panel, Element e, String ancestry) {
	if (!done(e)) {    
	    String text = e.getTextTrim();
	                
	    if (text.length()>0) {
		text = "<html><font color='#444444'><b>"+ancestry+"</b></font></html>";
		doTextContent(text,panel,e);
	    }
	}
    }

    /** A variant on doTextContent(JPanel,Element,String) for when
     * you want to specify the text used to label the input field.
     * @see #doTextContent(JPanel panel, Element e, String ancestry)
     */
    protected JTextField doTextContent(String label, JPanel panel, Element e) {
	panel.add ( new JLabel(label) );
	JTextField field = new JTextField(e.getText());
	panel.add ( field );
	field.addFocusListener( new ValueUpdater(e) );    
	return field;
    }
    
    /** Deals with attributes of the current element, eg
     *  <place id="p1"> - creates a label and a field,
     *  with attached Listener of class AttribUpdater.
     *  If the attribute is "id", sets the field to be non-editable.
     *  @see #AttribUpdater
     */
    protected void doAttributes(JPanel panel, Element e, String ancestry) {
	
	java.util.List attribs = e.getAttributes();
	String label;
	if (attribs.size()>0) {
	    if (ancestry.length()>1)
		ancestry = ancestry+".";
	                
	    Iterator as= attribs.iterator();
	    while (as.hasNext()) {
		Attribute a = (Attribute) as.next();
		if (!done(a)) { 
		    label = "<html><font color='#444444'>"+ancestry+"</font><font color='#000088'><b>"+a.getName()+"</b></font></html>";
		    doAttribute(label,panel,e,a);
		}
	    }
	}
    }

    /** A variant on (and helper for) doAttributes(JPanel,Element,String) 
     * for when you want to specify the text used to label the input field.
     * @see #doAttributes(JPanel panel, Element e, String ancestry)
     */
    protected JTextField doAttribute(String label, JPanel panel, Element e, Attribute a) {
	panel.add ( new JLabel(label));
	JTextField field = new JTextField(a.getValue());
	panel.add ( field );
	field.addFocusListener( new AttribUpdater(e,a.getName()) );
	return field;
    }

    /** A variant on (and helper for) doAttributes(JPanel,Element,String) 
     * for when you want to specify the text used to label the input field,
     * and you have a fixed set of allowed values for this attribute.
     * @see #doAttributes(JPanel panel, Element e, String ancestry)
     */
    protected JComboBox doAttribute(String label, JPanel panel, Element e, Attribute a, Vector values) {
	panel.add ( new JLabel(label));
	JComboBox field = new JComboBox(values);
	field.setSelectedItem(a.getValue());
	panel.add ( field );
	field.addFocusListener( new AttribUpdater(e,a.getName()) );
	return field;
    }
    
    protected boolean done(Object o){
	if (specials.contains(o)) return true;
	if (hiddens.contains(o)) return true;
	return false;
    }

    /** Override in subclasses to find and add to the panel
     * elements that require special treatment, and those that
     * should always be available for a particular type of petri
     * net object.
     */
    protected void doSpecial(JPanel panel, Element e) {
    } 

    /** Override in subclasses to find those elements which
     * should not be shown in the properties box.
     */
    protected void doHidden(JPanel panel, Element e) {
    } 

    protected String format(String text){
	return "<html><font color='#444499'><b>"+text+"</b></font></html>";
    }

    /* ----------------------------- classes -------------------- */    
    /** Listener class which is attached to an input text field
     * and updates the JDOM Element with any changes.
     * Uses setAttribute: this works whether the attribute
     * already exists or not.
     */
    public class AttribUpdater implements FocusListener {
	Element element;
	String attrib;
	
	public AttribUpdater(Element e, String att) {
	    element = e;
	    attrib = att;
	}
	
	public void focusGained(FocusEvent e) { }
	
	public void focusLost(FocusEvent e) {
	    if (!e.isTemporary()){
		try {
		    JTextField field = (JTextField) e.getComponent();
		    element.setAttribute(attrib, field.getText());
		} catch (Exception ex){
		    try {
			JComboBox field = (JComboBox) e.getComponent();
			element.setAttribute(attrib, (String) (field.getSelectedItem()));
		    } catch (Exception ex2){}
		        
		}

	    }
	}
    }

    /** Listener class which is attached to an input text field
     * and updates the JDOM Element with any changes.
     */    
    public class ValueUpdater implements FocusListener {
	Element element;
	
	public ValueUpdater(Element e) {
	    element = e;
	}
	
	public void focusGained(FocusEvent e) { }
	
	public void focusLost(FocusEvent e) {
	    if (!e.isTemporary()){
		JTextField field = (JTextField) e.getComponent();
		element.setText(field.getText());
	    }
	}
    }

   
}





