package uk.ac.ic.doc.y2002.tool.editor;

import javax.swing.JLabel;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.*;
import java.util.*;
import org.jdom.*;
import uk.ac.ic.doc.y2002.tool.application.*;

/** The RefLinkLabel class draws the reference links between
 *  reference places and places. Each reference is stored as
 *  an attribute of the reference place in the pnml
 */

public class RefLinkLabel extends JLabel {
    int sx,sy,tx,ty, ax, ay;
    int xmin,xmax,ymin,ymax; // bounds
    double rotation=0;
    PNLabel source, target;
    //   RefLabel line;

    public RefLinkLabel(PNLabel s, PNLabel t) {
	super();
	source = s;
	target = t;
	updateLine();
	calcBounds();
    }


    public void paintComponent(Graphics g){
	super.paintComponent(g);
	Graphics2D g2 = (Graphics2D)g;
	float[] dashPattern = { 5, 5};
	g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
	g2.setStroke(new BasicStroke(1.5f, BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER, 10,dashPattern, 0));
	Line2D line = new Line2D.Double(sx-getX(), sy-getY(), tx-getX(), ty-getY());
	this.updateLine();
	g2.draw(line);
	
	// draw the arrow, rotated
	calcRotation();
	// save original transform settings
	AffineTransform saveXform = g2.getTransform();
	// set up this Label's transform
	AffineTransform at = new AffineTransform();
	at.rotate(rotation,ax-getX(),ay-getY()); 
	g2.transform(at);
	g2.setStroke(new BasicStroke(1.0f));
	line = new Line2D.Double(ax-getX()-5, ay-getY()+10, ax-getX(), ay-getY());
	g2.draw(line);
	line = new Line2D.Double(ax-getX()+5, ay-getY()+10, ax-getX(), ay-getY());
	g2.draw(line);
	// restore original transform settings
	g2.setTransform(saveXform);
	
    }

    // remove RefLinkLabel from displayPane
    public DisplayPane remove(){

	// remove RefLinkLabel from DisplayPane it is in
	DisplayPane display = (DisplayPane) getParent();
	display.remove(this);
	
	return display;
    }

    public void updateLine() {
	sx = source.getJX();
	sy = source.getJY();
	tx = target.getJX();
	ty = target.getJY();
 
	// modified section.
	int dy = sy - ty;
	int dx = tx - sx;
	double cos_theta = (double)dx/(Math.sqrt((double)((dy*dy)+(dx*dx))));
	double sin_theta = (double)dy/(Math.sqrt((double)((dy*dy)+(dx*dx))));
	//	System.out.println("dy: " + dy + "dx: " + dx);
	double radians = (((double)dy / (double) dx));
	//	System.out.println("The andgle in radians is "+radians);
	double dsx = 20 * cos_theta;//Math.cos (radians); 
	double dsy = 20 * sin_theta;//Math.sin (radians);
	//	System.out.println("Sine: " + dsy + " Cosine: " + dsx);
	sx = sx + (int) dsx;
	sy = sy - (int) dsy;
	tx = tx - (int) dsx;
	ty = ty + (int) dsy;

	ax = (tx + sx)/2;
	ay = (ty + sy)/2;
	calcBounds();
    }
    

    protected void calcBounds() {
	xmin = Math.min(sx,tx);  xmin = Math.min(xmin,ax);
	xmax = Math.max(sx,tx);xmax = Math.max(xmax,ax);
	ymin = Math.min(sy,ty);ymin = Math.min(ymin,ay);
	ymax = Math.max(sy,ty);ymax = Math.max(ymax,ay);
	setBounds(xmin-10,ymin-10,xmax+10,ymax+10);
	
	/*xmin = Math.min(sx,tx);
	  xmax = Math.max(sx,tx);
	  ymin = Math.min(sy,ty);
	  ymax = Math.max(sy,ty);
	  setBounds(xmin,ymin,xmax,ymax);*/
    }
    
    protected void calcRotation() {
	double angle = Math.atan( ( (double)tx-(double)sx)/( (double)sy-(double)ty));
	//System.out.println("LineLabel::angle is "+angle+ " - "+arc.getID());
	
	// first check for cases where line is horizontal
	if ((angle>-1.58 && angle < -1.56) || (angle>1.56 && angle<1.58)){
	    //System.out.println("LineLabel:: +-1.57");
	    if (sx>tx){
		//System.out.println("LineLabel:: Source is right of target");
		angle = Math.toRadians(-90);
	    } else {
		//System.out.println("LineLabel:: Source is left of target");
		angle = Math.toRadians(90);
	    }
	// otherwise, determine rotation by quadrant
	} else {
	    if (tx>=sx && ty>=sy) {
		// rotate by 180 degrees
		angle += Math.toRadians(180);
	    } else if (sx>=tx && sy>=ty) {
		// ok as is
	    } else if (tx<sx && ty>=sy) {
		// rotate by 180 degrees
		angle += Math.toRadians(180);
	    } else if (sx<tx && sy>=ty) {
		// ok as is
	    }
	}
	rotation = angle;
    }

      
    /** Returns a list of PNLabels which are linked to this PNLabel. 
     * The list will consist of the Source label and the Target label. 
     */
    public java.util.List getLinkedLabels(){
	java.util.List linked = new java.util.LinkedList();
	linked.add(source);
	linked.add(target);
	return linked;
    }
    
}





















