package uk.ac.ic.doc.y2002.tool.editor;

import javax.swing.JLabel;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*; 
import java.awt.*;
import java.util.*;
import org.jdom.*;

public class ReferencePlaceLabel extends PlaceLabel implements MouseListener, MouseMotionListener{

    public ReferencePlaceLabel(Element e) {
	super(e);
	// set up member variables
	xoffset = radius + offset;
	yoffset = radius + offset;
	preferredSize = new Dimension(2*(radius+offset),2*(radius+offset));
	propertiesbox = NodePropertiesBox.class;
    }

    // Function to create a new referencePlace element.
    public static Element makeElement(Integer x, Integer y, String id, Namespace ns) {

	Element refPlace = new Element("referencePlace", ns);
	refPlace.setAttribute("id", id);

	Element graphics = PNLabel.makeGraphics(String.valueOf(x), String.valueOf(y), ns);
	Element name = NodeLabel.makeName(id, ns);
	
	refPlace.addContent(name);
	refPlace.addContent(graphics);
	
	return refPlace;
    }
    

    public void updateBounds() {
	setBounds(lastX-(radius+offset),lastY-(radius+offset),2*(radius+offset)+textspace,2*(radius+offset)+textspace);

    }


    public void paintComponent(Graphics g){
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D)g;
	g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
	// Draws the circle.
	g2.setStroke(new BasicStroke(2.0f));
	g2.setPaint(fillcolour);
	g2.fill (new Ellipse2D.Double(offset, offset, 2*radius, 2*radius));
	float[] dashPattern = { 5, 5};
	g2.setStroke(new BasicStroke(2, BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER, 10,dashPattern, 0));
	g2.setPaint(Color.black);
	g2.draw (new Ellipse2D.Double(offset, offset, 2*radius, 2*radius));
	drawTokens(g2);
	paintName(g2);
    }

    protected void paintName(Graphics2D g2){
	// Draws the name.
	String name = getName();
	g2.drawString(name,radius/3,2*(radius+offset*2));
	String id = getID();
	if (!name.equals(id)){
	    name = "("+id+")";
	    g2.drawString(name,radius/2,2*(radius+offset*2)+13);
	}
    }

    // Function to create a new referenceplace element.
    public static Element makeElement(int x, int y, String id, Namespace ns) {

	Element referencePlace = new Element("referencePlace", ns);
	referencePlace.setAttribute("id", id);
	Element graphics = PNLabel.makeGraphics(String.valueOf(x), String.valueOf(y), ns);
	Element name = NodeLabel.makeName(id, ns);
	
	referencePlace.addContent(name);
	referencePlace.addContent(graphics);
	
	return referencePlace;
    }

    /** Retrieves the PlaceLabel referenced */
    public PlaceLabel getPlace(){
	if (refNodeTarget==null){
	    //System.out.println("ReferencePlaceLabel:: no reference to a PlaceLabel stored!");
	    return null;
	} else {
	    try {
		PlaceLabel p = (PlaceLabel) refNodeTarget;
		return p;
	    } catch (Exception e) {
		//System.out.println("ReferencePlaceLabel:: reference stored is not to a PlaceLabel!");
		return null;
	    }
	}
	
    }

    /** Gets the Marking value from the place referenced
     */
    public int getMarking() {
	PlaceLabel p = getPlace();
	if (p!=null) return p.getMarking();
	else return 0;
    }

 
    /** Sets the Marking value in the place referenced
     */
    protected void setMarking(int t) {
	PlaceLabel p = getPlace();
	if (p!=null) p.setMarking(t);
    }
    
    /** Use this function to add a token to the place.
     *  As with all these functions, it passes the request
     *  onto the 'real' PlaceLabel.
     */
    public void addToken(){
	PlaceLabel p = getPlace();
	if (p!=null) p.addToken();
	updateReferences();
    }
    /** Use this function to add a specified number of tokens to the place.
     */
    public void addToken(int number){
	PlaceLabel p = getPlace();
	if (p!=null) p.addToken(number);
	updateReferences();
    }
    
    /** Use this function to remove a token from the place.
     */
    public void remToken(){
	PlaceLabel p = getPlace();
	if (p!=null) p.remToken();
	updateReferences();
    }

    /** Use this function to remove the specified number of tokens from the place.
     */
    public void remToken(int number){
	PlaceLabel p = getPlace();
	if (p!=null) p.remToken(number);
	updateReferences();
    }

    
}












