package uk.ac.ic.doc.y2002.tool.editor;

import javax.swing.JLabel;
import javax.swing.event.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*; 
import java.awt.*;
import java.util.*;
import org.jdom.*;
import uk.ac.ic.doc.y2002.tool.application.*;

/** Class used by editing tools to select objects and store cut and copied objects.
 */

public class SelectionBox extends JLabel {
    int height = 0, width = 0;   // default values for box before stretching.
    int xOffset, yOffset;
    int cutXOffset, cutYOffset;
    int xPress, yPress;
    Dimension preferredSize = new Dimension();
    LinkedList containedElements = new LinkedList(); // list of elements contained in SelectionBox
    BasicStroke dottedLine = new BasicStroke(2.0f, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 
					     10.0f, new float[] {5.0f, 5.0f}, 0.0f);
    BasicStroke normalLine = new BasicStroke(2.0f);
    BasicStroke lineStroke; // chosen line type.
    boolean beingMoved = false;

    public SelectionBox(int xOffsetArg, int yOffsetArg) {

	super();
	xOffset = xOffsetArg;
	yOffset = yOffsetArg;
	updateBounds();
	updatePreferredSize();
	lineStroke = dottedLine;
	
    }
   
    /** Get selection box height
     *  @return int
     */
    public int getHeight() {
	return height;
    }

    /** Get the elements contained by the box
     *  @return LinkedList
     */
    public LinkedList getContainedElements() {
	return containedElements;
    }

    /** stretch Selection Box */
    public void stretchBox(int widthArg, int heightArg) {

	height = heightArg;
	width = widthArg;
	updateBounds();
	updatePreferredSize();
    }

    /** Calculate stored labels */
    public void calculate(HashMap idTable) {

	// get rectangle for selectionBox
	Rectangle my_rect = getBounds();
	Rectangle label_rect;
	PNLabel label;

	// iterate through labels in id table - first pass for nodelabels
	Iterator it = idTable.entrySet().iterator();

	while (it.hasNext()) {
	    Map.Entry e = (Map.Entry) it.next();
	    label = (PNLabel) e.getValue();
	    label_rect = label.getBounds();
 
	    // select labels that intersect with selection box.
	    if (my_rect.intersects(label_rect) && (!label.returnElement().getName().equals("arc"))) {
		label.selectOnly(true);
		addContained(label);
	    }
	    // unselect others.
	    else 
		label.selectOnly(false);
	}

	// iterate through labels in id table - second pass for arclabels
	Iterator it2 = idTable.entrySet().iterator();

	while (it2.hasNext()) {
	    Map.Entry e = (Map.Entry) it2.next();
	    label = (PNLabel) e.getValue();
	    label_rect = label.getBounds();
	    
	    // select labels that intersect with selection box.
	    if (my_rect.intersects(label_rect) && (label.returnElement().getName().equals("arc"))) {
		
		if (checkLabel(label)) {
		    label.selectOnly(true);
		    addContained(label);
		    System.out.println(label.getID());
		}
	    }
	}
	
    }
  
    private boolean checkLabel(PNLabel label) {

	boolean selected = true;
	java.util.List linked = label.getLinkedLabels();
	Iterator it = linked.iterator();

	while (it.hasNext()) {
	    
	    PNLabel connected = (PNLabel) it.next();
	    if (!connected.returnSelected())
		selected = false;
	}

	return selected;
    }

    // add contained labels with cloned elements.
    private void addContained(PNLabel label) {
	      
	Element cloneElement = (Element) label.returnElement().clone();
	containedElements.add(cloneElement);

    }

    /** paste SelectionBox into top corner of displayPane */
    public void paste() {

	xOffset = yOffset = 5;
	updateBounds();
	
	addMouseListener( new MouseAdapter() {
		public void mousePressed(MouseEvent e) {
		    beingMoved = true;
		    xPress = e.getX();
		    yPress = e.getY();
		    // on double click paste elements.
		    if (e.getClickCount() == 2) {
			pasteElements();
			removeMe();
		    }
		}

		public void mouseReleased(MouseEvent e) {
		    beingMoved = false;
		    updateLocation(e);
		}
	    });

	addMouseMotionListener( new MouseMotionAdapter() {
		public void mouseDragged(MouseEvent e) {     
		    if (beingMoved)
			updateLocation(e);
		}
	    });
    }

    // paste elements onto display.
    private void pasteElements() {

	DisplayPane display = (DisplayPane) getParent();

	// paste node elements
	Iterator it = containedElements.iterator();
	
	while (it.hasNext()) {
	    Element e = (Element) it.next();
	    //update x and y values.
	    updateElementLocation(e);
	    if (!e.getName().equals("arc"))
		display.addNode(e);
	}
	 
	// paste arc elements
	Iterator it2 = containedElements.iterator();
	
	while (it2.hasNext()) {
	    Element e = (Element) it2.next();
	    if (e.getName().equals("arc"))
		display.addArc(e);
	}
    }

    // update element x and y to paste location.
    private void updateElementLocation(Element e) {

	int oldX, oldY;
	
	oldX = getJX(e);
	oldY = getJY(e);

	setJX(e, oldX + xOffset -cutXOffset);
	setJY(e, oldY + yOffset -cutYOffset);
    }
    
      // get X, Y values from the JDOM Document
    private int getJX(Element e) {
	Namespace ns = e.getNamespace();

	return Integer.parseInt(e.getChild("graphics",ns).getChild("position",ns).getAttributeValue("x"));
    }
    private int getJY(Element e) {
	Namespace ns = e.getNamespace();

	return Integer.parseInt(e.getChild("graphics",ns).getChild("position",ns).getAttributeValue("y"));
    }

    // set X, Y values in the JDOM Document
    // note this may need conversion from the actual x,y values for the label
    protected void setJX(Element element, int x) {
	Namespace ns = element.getNamespace();
	
	if (element.getChildren("graphics",ns).size()>0   /* avoid NULL exceptions */
	    && element.getChild("graphics",ns).getChildren("position",ns).size()>0 ) { 
	    element.getChild("graphics",ns).getChild("position",ns).setAttribute("x", String.valueOf(x));
	}
    }
    protected void setJY(Element element, int y) {
	Namespace ns = element.getNamespace();
	
	if (element.getChildren("graphics",ns).size()>0   /* avoid NULL exceptions */
	    && element.getChild("graphics",ns).getChildren("position",ns).size()>0 ) { 
	    element.getChild("graphics",ns).getChild("position",ns).setAttribute("y", String.valueOf(y));
	}
    }

    /** remove selection box from DisplayPane */
    public void removeMe() {

	DisplayPane display = (DisplayPane) getParent();
	display.remove(this);
	display.repaint();
    }

    /** store position at which box is cut */
    public void setCutOffset() {

	// save cut position.
	cutXOffset = xOffset;
	cutYOffset = yOffset;
    }
    
    /** in response to a pasted object being dragged event location is updated */
    protected void updateLocation(MouseEvent e){
       
	xOffset = xOffset + e.getX() - xPress;
	yOffset = yOffset + e.getY() - yPress;
       
	updateBounds();
    }
	      
    private void updateBounds() {
  
	setBounds(xOffset, yOffset, width, height);
    }
     
    private void updatePreferredSize() {

        preferredSize.setSize(width, height);
    }

    public void paintComponent(Graphics g){
        super.paintComponent(g);
	Graphics2D g2 = (Graphics2D)g;
	    
	// Draws the selection box.
	g2.setStroke(lineStroke);
	g2.draw(new Rectangle2D.Double(0, 0, width, height));
       
    }
}













