package uk.ac.ic.doc.y2002.tool.editor;

import javax.swing.JLabel;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.*;
import java.util.*;
import org.jdom.*;
import uk.ac.ic.doc.y2002.tool.application.*;

/** Class for internal export labels in subnet instance.
 *  Should be added to the subnet label not displayPane.
 */
public class SubnetExportLabel extends ExportPlaceLabel implements MouseListener, MouseMotionListener{

    private SubnetLabel parent;
    private DisplayPane my_display;
    private int xPos = 55, yPos = 5;

    public SubnetExportLabel(Element e, SubnetLabel parent) {
	super(e);
	this.parent = parent;
    }

    // overidden values for size and bounds
    public void updateBounds() {

	setBounds(xPos, yPos, 50, 50+textspace);
    }

    // overidden values for position (set relative to SubnetLabel).
    public void setPosition(int x, int y) {

	setLocation(xPos,yPos);
    }

    // overridden mouse methods for internal module label.
    public void mousePressed(MouseEvent e){

	if (Application.setTool.equals("pointer")) {
	    parent.mousePressed(e);
	}
	else if (Application.setTool.equals("ref")) {
	    DisplayPane display = (DisplayPane) parent.getParent();
	    display.notify(this);
	}
    }
    public void mouseDragged(MouseEvent e){

	if (Application.setTool.equals("pointer"))
	    parent.mouseDragged(e);
    }
    public void mouseReleased(MouseEvent e){

	if (Application.setTool.equals("pointer"))
	    parent.mouseReleased(e);
    }

    public void mouseClicked(MouseEvent e) {}


    /** Method to remove an internal label from a subnet. Used by parent of
     *  of element to remove itself.
     */
    public void removeLabel(){

	DisplayPane display = getDisplayPane();

	// remove element from JDOM.
	Element my_parent = element.getParent();
	my_parent.removeContent(element);

	// ask PNLabel's reference links to remove themselves if they exist.
	String referenceID = element.getAttributeValue("ref");
	String elementName = element.getName();

	if (referenceID != null) {
	    removeReferenceLabel();
	    removeTargetSourceEntry(this);
	}

	// if target for a reference link.
	if (referenceTarget()) {
	    notifySource();
	    java.util.List sourceList = new LinkedList(getRefNodeSources());
	    Iterator it = sourceList.iterator();

	    while (it.hasNext()) {
		NodeLabel source = (NodeLabel) it.next();
		source.removeRefNodeTarget();
		removeRefNodeSource(source);
	    }
	}

	// remove PNLabel from DisplayPane it is in.
	parent.remove(this);
    }

    /** Method to get DisplayPane in which the parent of these elements is contained
     *@return The DisplayPane which contains this PNLabel.
     */
    public DisplayPane getDisplayPane(){
	// select the parent JDesktopFrame and add the Properties popup to it
	SubnetLabel subnet = (SubnetLabel) getParent();
	Container con = subnet.getParent();
	Class c = con.getClass();
	if (c == DisplayPane.class) {
	    //    System.out.println("PNLabel::got parent display pane");
	    return (DisplayPane) con;
	} else {
	    // shouldn't happen!
	    System.out.println("PNLabel::Error - can't find parent display pane!");
	    return null;
	}
    }



    /** Function to create a new exportplace element for use in a subnet.
     *  @return Element
     */
    public static Element makeElement(String parameter,String instance, String ref, Namespace ns) {

	Element exportPlace = new Element("exportPlace", ns);
	exportPlace.setAttribute("parameter", parameter);

	// if an instance is set.
	if (instance != null)
	    exportPlace.setAttribute("instance", instance);

	// if a reference is set.
	if (ref != null)
	    exportPlace.setAttribute("ref", ref);
	
	return exportPlace;
    }
      
    /** Gets the name (attribute is  parameter) of the subnetExportLabel
     *  @return String
     */
    public String getName() {
 
	String name = element.getAttributeValue("parameter");

	if (name == null)
	    return "";
	return name;
    }
    
    // overidden PNLabel method 
    public int getJY() {
	
	if (parent != null && parent.element != null) {
	    return parent.getY() + yPos + 20 + 4;
	}
	else 
	    return 0;
    }

    // overidden PNLabel method
    public int getJX() {
	
	if (parent != null && parent.element != null) {
	    return parent.getX() + xPos + 20 + 4;
	}
	else 
	    return 0;
    }
    
    /** Gets the ID of an element. To identify label ID returned 
     *  //is subnet_id + "." + my_id
     *  @return String
     */
    public String getID() {

	String my_id = element.getAttributeValue("parameter");
	return my_id;
	//String subnet_id = parent.element.getAttributeValue("id");
	//return subnet_id + "." + my_id;
    }
}










