package uk.ac.ic.doc.y2002.tool.editor;

import javax.swing.JLabel;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*; 
import javax.swing.*;
import java.awt.*;
import java.util.*;
import org.jdom.*;
import java.io.*;
import org.jdom.input.*;
import org.jdom.output.*;
import uk.ac.ic.doc.y2002.tool.application.*;

/** SubnetLabel represents the instance of a subnet within a petri net
 *  It creates labels for import and export places within itself
 */


public class SubnetLabel extends NodeLabel implements MouseListener, MouseMotionListener{
    public static Element chosenRoot = null;
    public static String chosenRef = null;
    int height = 60;
    int width = 100;
    int radius = 20;
    Element moduleRoot = null;
    SubnetImportLabel importLabel;
    SubnetExportLabel exportLabel;
    Document instance;
    public EditorFrame instanceFrame;
    Color enabledfill = new Color((float)0.5,(float)0.5,(float)1);    
    Color firefill = Color.red;    
    
    public SubnetLabel(Element e) {
	super(e);
	// set up member variables
	xoffset = width/2;
	yoffset = height/2;
	preferredSize = new Dimension(width+offset*2,height+offset*2+textspace);
	updateBounds();
	// do custom initialisation
	String imp, exp;
	contextmenu = SubnetContextMenu.class;
	newExportImport();
    }

    /** Remove a subnet from the displayPane in which it is contained
     *  @return DisplayPane
     */
    public DisplayPane remove(){
       
	DisplayPane display = getDisplayPane();
	
	// remove element from JDOM.
	Element my_parent = element.getParent();
	my_parent.removeContent(element);
	
	// remove export and import labels.
	importLabel.removeLabel();
	exportLabel.removeLabel();

	// remove PNLabel from DisplayPane it is in.
	display.remove(this);

	return display;
    }
    /** Draws import and export places within the subnet instance
     */
    private void newExportImport() {

	// get import and export elements from jdom
	try{
	    Element importElement = element.getChild("importPlace", ns);
	    Element exportElement = element.getChild("exportPlace", ns);

       
	    importLabel = new SubnetImportLabel(importElement, this);
	    exportLabel = new SubnetExportLabel(exportElement, this);

	    // add labels to subnet.
	    add(importLabel);
	    add(exportLabel);
	}catch(Exception err){System.out.println("SubnetLabel :: Instance does not contain import and export places");}
    }

    /** Open the template corresponding to the subnet instance
     */
    public void openSubnetTemplate() {
	System.out.println("SubnetLabel:: opening subnet template");
	File myFile = new File(this.getFilePath());

	// Request document building without validation.
	SAXBuilder builder = new SAXBuilder(false);

	try {
	    Document doc = builder.build(myFile);
	    examineDoc(doc, myFile, false);
	}
	catch (Exception error) { System.out.println("Error - file not read."); }
    }

    /** Create and open an instance of this subnet
     */
    public void openSubnetInstance() {
	System.out.println("SubnetLabel:: opening subnet instance");
	instanceFrame.setVisible(true);
	try {
	    instanceFrame.setSelected(true);
	} catch (java.beans.PropertyVetoException e) {};    
    }

    /** Hide instance of this subnet
     */
    public void closeSubnetInstance() {
	instanceFrame.setVisible(false);    
    }
    
    /** Generates subnet instance from module template,
     *  and creates a new window to display it, with accompanying
     *  PNLabels to enable animation - doesn't make it visible though.
     */
    public void createSubnetInstance() {
	System.out.println("SubnetLabel:: creating subnet instance "+getID());
	Element module = getTemplateModule();
	Element net = (Element) module.clone();
	if (net==null) return;
	// --- set top element to <net> rather than <module>
	net.setName("net");
	// --- make new Document
	Namespace ns = net.getNamespace();
	Element root = new Element("pnml",ns);
	root.addContent(net);
	Document newDoc = new Document(root);
	// --- append "instance id" to end of name
	if (net.getChildren("name",ns).size()<=0)
	    net.addContent(new Element("name",ns));
	if (net.getChild("name",ns).getChildren("value",ns).size()<=0)
	    net.getChild("name",ns).addContent(new Element("value",ns));
	String netname = net.getChild("name",ns).getChildTextTrim("value",ns);
	netname += " - instance "+getID();
	net.getChild("name",ns).getChild("value",ns).setText(netname);
	// ---
	
	examineDoc(newDoc,null,true);    
    }
    
    // Method to examine loaded JDOM document.
    private void examineDoc(Document doc, File myFile, boolean isInstance) {

	Element root = doc.getRootElement();
	Namespace ns = root.getNamespace();
	java.util.List net_content = root.getChildren("module", ns);
	java.util.List net_content2 = root.getChildren("net", ns);
	if (net_content.size()>0 && net_content2.size()>0)
	    net_content.addAll(net_content2);
	else if (net_content.size()==0 && net_content2.size()>0)
	    net_content = net_content2;
	Iterator it = net_content.iterator();

	// create new frame on desktop for each net
	while (it.hasNext()) {
	    Application app = this.getDisplayPane().eframe.app;
	    Element element = (Element) it.next();
	    // remove old instanceFrame if it exists
	    if (isInstance && instanceFrame!=null){
		System.out.println("Close existing subnet frame");
		instanceFrame.dispose();
	    }
	    EditorFrame frame = new EditorFrame(app, doc, element, myFile);
	    app.desktop.add(frame);
	    if (isInstance) {
		instanceFrame = frame;
		frame.display.linkSubnet(importLabel, exportLabel);
		frame.subnet = this;
	    }   else frame.setVisible(true);
	}

    }

    /** Gets the path where the subnet template is stored
     */
    private String getFilePath() {
	String p = element.getAttributeValue("ref");
	System.out.println(p);
	int end = p.indexOf('#');
	System.out.println(end);
	String path = p.substring(0, end);
	System.out.println(path);
	return path;    
    }
    /** Returns the <module> element of the subnet template
     */
    public Element getTemplateModule(){
	Element module = null;
	File myFile = new File(this.getFilePath());

	// Request document building without validation.
	SAXBuilder builder = new SAXBuilder(false);

	try {
	    Document doc = builder.build(myFile);
	    Element root = doc.getRootElement();
	    Namespace ns = root.getNamespace();
	    module = root.getChild("module", ns);
	}
	catch (Exception error) { System.out.println("Error - file not read."); }
	
	return module;    
    }
    /** Returns the import or export label corresponding to the input id
     */
    public Object getImpExpLabel(String id) {
	Object o = null;
	if (importLabel.element.getAttributeValue("parameter").equals(id))
	    o = importLabel;
	else if (exportLabel.element.getAttributeValue("parameter").equals(id))
	    o = exportLabel;
	else{
	    System.out.println("SubnetLabel:: could not find import/ export place " + id);
	}
	return o;
    }
    /** Returns a string ref identifying the template that the user has  chosen
     *  to insert into the net
     */
    public static String chosenRef(){
	if (chosenRef == null){
	    return "error";
	}
	return chosenRef;
    }
    /** Returns the id of the importplace of the template that been chosen
     *  to insert
     */  
    public static String importPlace(){
	Namespace ns = chosenRoot.getNamespace();
	try{
	    return chosenRoot.getChild("interface", ns).getChild("importPlace", ns).getAttributeValue("id");
	}catch(Exception e2){System.out.println("SubnetLabel.importPlace:: " + e2);}
	return "oops";
    }

    /** Returns the id of the exportplace of the template that been chosen
     *  to insert
     */  

    public static String exportPlace(){
	Namespace ns = chosenRoot.getNamespace();
	try{
	    return chosenRoot.getChild("interface", ns).getChild("exportPlace", ns).getAttributeValue("id");
	}catch(Exception e2){System.out.println("SubnetLabel.importPlace:: " + e2);}
	return "oops";
    }


    /** Function to create a new subnet element.
     */
    public static Element makeElement(Integer x, Integer y, String id, String moduleRef,
				      String importParameter, String exportParameter, Namespace ns) {

	Element instance = new Element("instance", ns);
	instance.setAttribute("id", id);
	try{
	    instance.setAttribute("ref", moduleRef);
	}catch(NullPointerException e) {
	    System.out.println("Unable to set module ref");
	} 
	Element graphics = PNLabel.makeGraphics(String.valueOf(x), String.valueOf(y), ns);
	Element name = NodeLabel.makeName(id, ns);

	// create new import and export place elements and labels.
	Element importElement = SubnetImportLabel.makeElement(importParameter, null, null, ns);
	Element exportElement = SubnetExportLabel.makeElement(exportParameter , null, null, ns);

	instance.addContent(name);
	instance.addContent(graphics);
	instance.addContent(importElement);
	instance.addContent(exportElement);

	return instance;
    }

    public void updateBounds() {
	setBounds(lastX-(width/2+offset),lastY-(height/2+offset), width+offset*2,height+offset*2+textspace);
    }
    
    public void paintComponent(Graphics g){
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D)g;
	
	// save original transform settings
	AffineTransform saveXform = g2.getTransform();
	// set up this Label's transform
	AffineTransform at = new AffineTransform();
	at.scale(scaling, scaling); 
	at.rotate(Math.toRadians(rotation),offset+(height-width),offset+height/2); 
	g2.transform(at);
	
	g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
	// Draws the rectangle.
	g2.setStroke(new BasicStroke(2.0f));
	g2.draw(new Rectangle2D.Double(offset ,offset, width, height));
	g2.setPaint(fillcolour);
	g2.fill (new Rectangle2D.Double(offset, offset, width, height));
	g2.setPaint(Color.black);
	
	// restore original transform settings
	g2.setTransform(saveXform);
	
	// Draws the name.
	String name = getName();
	g2.drawString(name,offset*2,height+offset*4);
	
	String templateName = getTemplateName();
	g2.drawString(templateName, (offset*2) + (width/3) + 4 , offset*2 +6 );
    }

    /** Gets the name of a subnetTemplate
     *  @return String
     */
    private String getTemplateName() {
	String ref = element.getAttributeValue("ref");
	int i = ref.indexOf('#');
	return ref.substring(i+1);
    }
    
    public void highlightFire(){
	fillcolour = firefill;
	repaint();
    }    
    public void highlightEnabled(boolean highlight){
	fillcolour = enabledfill;
	repaint();
    }    

}












