package uk.ac.ic.doc.y2002.tool.editor;

import javax.swing.JLabel;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*; 
import java.awt.*;
import java.util.*;
import org.jdom.*;

/** The TransitionLabel represents a normal transition.
 * It has a custom right-click menu which allows the user to
 * fire or rotate the transition as well as allowing the usual
 * access to properties.
 */
public class TransitionLabel extends NodeLabel implements MouseListener, MouseMotionListener{
    int height = 36;
    int width = 16;
    Color enabledfill = new Color((float)0.5,(float)0.5,(float)1);
    Color firefill = Color.red;

    public TransitionLabel(Element e) {
	super(e);
	// set up member variables
	xoffset = height-width;
	yoffset = height/2;
	preferredSize = new Dimension(width+offset*2,height+offset*2+textspace);
	// do custom initialisation
	height=36;
	width=16;
	contextmenu = TransitionContextMenu.class;
	fillcolour = Color.black;
	normalfill = Color.black;
	highlightfill = new Color((float) 0.4,(float) 0.4,(float) 0.7);
    }

    public void updateBounds() {
	setBounds(lastX-(height/2+offset),lastY-(height/2+offset),height+offset*2,height+offset*2+textspace);
    }
    
    public void paintComponent(Graphics g){
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D)g;
	
	// save original transform settings
	AffineTransform saveXform = g2.getTransform();
	// set up this Label's transform
	AffineTransform at = new AffineTransform();
	at.scale(scaling, scaling); 
	at.rotate(Math.toRadians(getRotation()),offset+(height-width),offset+height/2); 
	g2.transform(at);
	
	g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
	// Draws the rectangle.
	g2.setStroke(new BasicStroke(2.0f));
	g2.draw(new Rectangle2D.Double(offset+(height-width)/2,offset, width, height));
	g2.setPaint(fillcolour);
	g2.fill (new Rectangle2D.Double(offset+(height-width)/2,offset, width, height));
	g2.setPaint(Color.black);
	
	// restore original transform settings
	g2.setTransform(saveXform);
	
	// Draws the name.
	String name = getName();
	g2.drawString(name,offset*2,height+offset*4);
	String id = getID();
	if (!name.equals(id)){
	    name = "("+id+")";
	    g2.drawString(name,offset+7,height+offset*4+13);
	}
    }
    
    
    // Function to create a new transition element.
    public static Element makeElement(Integer x, Integer y, String id, Namespace ns) {
	
	Element transition = new Element("transition", ns);
	transition.setAttribute("id", id);

	Element graphics = PNLabel.makeGraphics(String.valueOf(x), String.valueOf(y), ns);
	Element name = NodeLabel.makeName(id, ns);
	
	transition.addContent(name);
	transition.addContent(graphics);
	
	return transition;
    } 

    /** Performs the fire by subtracting tokens from the input places
     * and adding them to the output places. This is done through the
     * input and output arcs. Changes are reflected in the JDOM Document.
     * This functions checks that the transition is active before firing.
     */
    public void fire(){
	// abort if transition is not active
	if (!isActive()){
	    System.out.println("TransitionLabel:: Can't fire - not active!");
	    return;
	}
	// fire transition : unfire all the input arcs
	Iterator inputs = inputArcs.iterator();
	while (inputs.hasNext()) {
	    ArcLabel arc = (ArcLabel) inputs.next();
	    arc.unfire();
	}

	// fire transition : fire all the output arcs
	Iterator outputs = outputArcs.iterator();
	while (outputs.hasNext()) {
	    ArcLabel arc = (ArcLabel) outputs.next();
	    arc.fire();
	}
	
    }

    /** Shortcut to allow the user to fire a transition by double-clicking it. */
    public void doubleclickAction(){
	fire();
    }

    public void highlightEnabled(boolean highlight){
	if (highlight)
	    fillcolour = enabledfill;
	else fillcolour = normalfill;
	selected = highlight;
	repaint();
	getDisplayPane().highlightEnabled(highlight);
    }
    public void highlightFire(){
	fillcolour = firefill;
	repaint();
	getDisplayPane().highlightFire();
    }

    /** Gets the Rotation property in the JDOM Document (default 0).
     * This property is toolspecific.
     */
    public int getRotation(){
	Element toolspecific = getToolspecific();
	if (toolspecific!=null && toolspecific.getChildren("rotation",ns).size()>0) {
	    String rotation = element.getChild("toolspecific",ns).getChildText("rotation",ns);
	    return Integer.parseInt(rotation);
	}
	return 0; // default if element not found in the Document
    }
    
    /** Sets the Rotation property in the JDOM Document.
     * This property is toolspecific.
     */
    public void setRotation(int rotation){
	Element toolspecific = getToolspecific();
	if (toolspecific==null)
	    toolspecific = makeToolspecific();
	if (toolspecific.getChildren("rotation",ns).size()<=0) {  
	    toolspecific.addContent(new Element("rotation",ns));
	}
	toolspecific.getChild("rotation",ns).setText(String.valueOf(rotation));
    }
}













