package uk.ac.ic.doc.y2002.tool.editor;

import javax.swing.JLabel;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*; 
import java.awt.*;
import java.util.*;
import org.jdom.*;

/** The TransitionStochasticLabel may represent Timed or
 * Immediate transitions.
 */
public class TransitionStochasticLabel extends TransitionLabel implements MouseListener, MouseMotionListener{

    public TransitionStochasticLabel(Element e) {
	super(e);
	propertiesbox = TransitionStochasticPropertiesBox.class;
	setColour();
    }

    
    // Function to create a new transition element.
    /*
    public static Element makeElement(Integer x, Integer y, String id, Namespace ns, String type, String weight) {
    
    Element transition = new Element("transition", ns);
    transition.setAttribute("id", id);

    Element graphics = PNLabel.makeGraphics(String.valueOf(x), String.valueOf(y), ns);
    Element name = NodeLabel.makeName(id, ns);

    Element t = new Element("type",ns);
    t.setAttribute("name",type);
    Element w = new Element("weight",weight);
    t.addContent(w);

    transition.addContent(name);
    transition.addContent(graphics);
    transition.addContent(t);
    
    return transition;
    } 
    */

    // Function to create a new transition element,
    // with default choice of immediate transition, weight 1
    /*
    public static Element makeElement(Integer x, Integer y, String id, Namespace ns) {
    
    Element transition = new Element("transition", ns);
    transition.setAttribute("id", id);

    Element graphics = PNLabel.makeGraphics(String.valueOf(x), String.valueOf(y), ns);
    Element name = NodeLabel.makeName(id, ns);

    Element type = new Element("type",ns);
    type.setAttribute("name","immediate");
    Element weight = new Element("weight","1");
    type.addContent(weight);

    transition.addContent(name);
    transition.addContent(graphics);
    transition.addContent(type);
    
    return transition;
    } 
    */

    /** Performs the fire by subtracting tokens from the input places
     * and adding them to the output places. This is done through the
     * input and output arcs. Changes are reflected in the JDOM Document.
     * This functions checks that the transition is active before firing.
     */
    public void fire(){
	// abort if transition is not active
	if (!isActive()){
	    System.out.println("TransitionLabel:: Can't fire - not active!");
	    return;
	}
	// fire transition : unfire all the input arcs
	Iterator inputs = inputArcs.iterator();
	while (inputs.hasNext()) {
	    ArcLabel arc = (ArcLabel) inputs.next();
	    arc.unfire();
	}

	// fire transition : fire all the output arcs
	Iterator outputs = outputArcs.iterator();
	while (outputs.hasNext()) {
	    ArcLabel arc = (ArcLabel) outputs.next();
	    arc.fire();
	}
	
    }

    /** Default version of setImmediate(float weight) */
    public void setImmediate(){
	setImmediate(getWeight());
    }
    /** Set the transition to be an immediate transition
     * with the given weight.
     */
    public void setImmediate(float weight){
	setType("immediate");
	setWeight(weight);
	setColour();
    }
    /** Default version of setTimed(float weight) */
    public void setTimed(){
	setTimed(getWeight());
    }
    /** Set the transition to be a timed transition
     * with the given weight.
     */
    public void setTimed(float weight){
	setType("timed");
	setWeight(weight);
	setColour();
    }

    public void setColour(){
	if (isImmediate()){
	    normalfill = Color.black;
	    highlightfill = new Color((float) 0.4,(float) 0.4,(float) 0.7);
	} else {
	    normalfill = Color.white;
	    highlightfill = new Color((float) 0.7,(float) 0.7,(float) 1);
	}
	fillcolour = normalfill;
    }

    public boolean isImmediate(){
	if (getType().equals("immediate")) return true;
	return false;
    }
    public boolean isTimed(){
	if (getType().equals("timed")) return true;
	return false;
    }
    
    /** Gets the transition weight from the JDOM Document. 
     * Defaults to 1 if not specified.
     */
    public float getWeight() {
	if (element.getChildren("weight",ns).size()>0) {
	    String weight = element.getChild("weight",ns).getText();
	    if (!weight.equals("")){
		float w = Float.parseFloat(weight);
		return w;
	    }
	}
	return 1; // if unspecified, return 1
    }

    /** Access function for the transition type stored in the JDOM Document:
     * <transition><type name="timed" /></transition> for example.
     * Defaults to "immediate" if unspecified.
     * @return The Transition Type (default "immediate")
     */
    public String getType(){
	// if <type> not present, assume default transition type "immediate".
	String type = "immediate";
	
	if (element.getChildren("type",ns).size()>0) {
	    if (element.getChild("type",ns).getAttributeValue("name")!=null) {
		type = element.getChild("type",ns).getAttributeValue("name");
		if (!type.equals("")) return type;
		else type = "immediate";
	    }
	}
	return type;
    }

    /** Sets the Transition Type in the JDOM Document.
     */
    protected void setType(String type){
	if (element.getChildren("type",ns).size()<=0) {
	    element.addContent(new Element("type",ns));
	}
	element.getChild("type",ns).setAttribute("name",type);
    }

    /** Sets the transition weight (aka priority) in the JDOM Document. */
    protected void setWeight(float weight){
	if (element.getChildren("weight",ns).size()<=0) {
	    element.addContent(new Element("weight",ns));
	}
	element.getChild("weight",ns).setText(String.valueOf(weight));
    }

    /** Function which allows the label to update itself (eg
     * for modifying screen display) in response to changes
     * in the underlying JDOM Element. Override in subclasses.
     */
    public void updateSelf(){
	setColour();
	repaint();
    }
   
}













